#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
import os
import cherrypy
from sponge import template
from utils import assert_raises

templates = os.path.abspath(os.path.join(os.path.dirname(__file__), 'templates'))

def test_render_html_param_filename_takes_string():
    assert_raises(TypeError, template.render_html, None, None,
                  exc_pattern=r'sponge.template.render_html ' \
                  'takes a string as filename param, got None.')
    assert_raises(TypeError, template.render_html, 5, None,
                  exc_pattern=r'sponge.template.render_html ' \
                  'takes a string as filename param, got 5.')

def test_render_html_param_filename_should_not_be_empty():
    assert_raises(TypeError, template.render_html, '', None,
                  exc_pattern=r'sponge.template.render_html ' \
                  'filename param can not be empty.')

def test_render_html_complains_cherrypy_not_configured_when_no_template_path_specified():
    assert_raises(LookupError, template.render_html, 'index.html', {},
                  exc_pattern=r'You must configure "template.dir" ' \
                  'string in CherryPy or pass template_path param ' \
                  'to render_html')

def test_render_html_param_template_path_takes_string():
    assert_raises(TypeError, template.render_html, 'index.html', {},
                  template_path=1,
                  exc_pattern=r'sponge.template.render_html ' \
                  'takes a string as template_path param, got 1.')
    assert_raises(TypeError, template.render_html, 'index.html', {},
                  template_path={},
                  exc_pattern=r'sponge.template.render_html ' \
                  'takes a string as template_path param, got {}.')

def test_render_html_param_template_path_takes_string():
    assert_raises(TypeError, template.render_html, 'index.html', {},
                  template_path=1,
                  exc_pattern=r'sponge.template.render_html ' \
                  'takes a string as template_path param, got 1.')
    assert_raises(TypeError, template.render_html, 'index.html', {},
                  template_path={},
                  exc_pattern=r'sponge.template.render_html ' \
                  'takes a string as template_path param, got {}.')

def test_render_html():
    got = template.render_html('test1.html', dict(title='foo', header='bar'), template_path=templates)
    assert '<title>My title: foo</title>' in got
    assert '<h1>My header: bar</h1>' in got



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# <Sponge - Lightweight Web Framework>
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
# Copyright (C) 2009 Bernardo Heynemann <heynemann@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import re
import sys
import cherrypy

from sponge.core.io import FileSystem, ClassLoader

class InvalidValueError(Exception):
    pass

class RequiredOptionError(Exception):
    pass

class ConfigValidator(object):
    class AnyValue(object):
        def __init__(self, vartype):
            if not isinstance(vartype, type):
                raise TypeError, 'ConfigValidator.AnyValue takes a ' \
                  'type as parameter, got %s' % repr(vartype)
            self.vartype = vartype

    mandatory = [
        'run-as',
        'host',
        'port',
        'autoreload',
        'application',
    ]
    possible = {
        'run-as': ['standalone', 'wsgi'],
        'host': r'^\d{1,3}[.]\d{1,3}[.]\d{1,3}[.]\d{1,3}$',
        'port': r'^\d+$',
        'autoreload': AnyValue(bool),
        'application': {
            r'^[a-zA-Z_-][\w_-]*$': r'^[/].*$'
        },
        'static': {
            r'^[a-zA-Z/_-][\w_-]*$': r'^[/].*$'
        },
        'databases': {
            r'^[\w_-]+$': '^.+$'
        },
        'extra': AnyValue(dict)
    }

    def __init__(self, cdict):
        if not isinstance(cdict, dict):
            raise TypeError, 'ConfigValidator takes a dict as parameter, got None.'
        self.cdict = cdict

    def validate(self):
        self.validate_mandatory()

    def raise_invalid(self, option, value):
        raise InvalidValueError, 'Invalid value in "%s" ' \
              'option: "%s". Read the Sponge documentation ' \
              'for more information.' % (option, value)

    def validate_mandatory(self):
        keys = self.cdict.keys()
        for option in self.possible:
            if option in self.mandatory and \
                option not in keys:
                raise RequiredOptionError, \
                      'You get to set "%s" option within settings.yml' % option
            if option not in self.cdict.keys():
                continue

            validator = self.possible[option]
            raw_value = self.cdict[option]
            value = unicode(raw_value)
            if isinstance(validator, list):
                if value not in validator:
                    self.raise_invalid(option, value)

            if isinstance(validator, basestring):
                if not re.match(validator, value):
                    self.raise_invalid(option, value)

            if isinstance(validator, self.AnyValue):
                if not isinstance(raw_value, validator.vartype):
                    self.raise_invalid(option, value)

            if isinstance(validator, dict):
                if not isinstance(raw_value, dict):
                    self.raise_invalid(option, value)
                self.validate_dict(option, validator, raw_value)

        return True

    def validate_dict(self, option, validator, dict_to_validate):
        for key_regex, value_regex in validator.items():
            for key, value in dict_to_validate.items():
                if isinstance(value, dict):
                    self.validate_dict(key, validator, value)

                elif isinstance(value, basestring):
                    if not re.match(key_regex, key):
                        self.raise_invalid(option, key)
                    if not re.match(value_regex, value):
                        self.raise_invalid(key, value)

                else:
                    self.raise_invalid(key, value)

class SpongeConfig(object):
    fs = FileSystem()

    def __init__(self, config_dict, validator):
        if not isinstance(config_dict, dict):
            raise TypeError, 'SpongeConfig parameter 1 must be a dict, ' \
                  'got %s.' % repr(config_dict)

        if not isinstance(validator, ConfigValidator):
            raise TypeError, 'SpongeConfig parameter 2 must be a ' \
                  'ConfigValidator, got %s.' % repr(validator)

        self.d = config_dict
        self.validator = validator

    def set_setting(self, key, value):
        self.d[key] = value

    def setup_all(self, current_full_path):
        if not isinstance(current_full_path, basestring):
            raise TypeError, 'SpongeConfig.setup_all takes a string, ' \
                  'got %s.' % repr(current_full_path)

        if not os.path.isabs(current_full_path):
            raise TypeError, 'SpongeConfig.setup_all takes a absolute ' \
                  'path, got %s.' % current_full_path

        self.set_setting('server.socket_port', int(self.validator.cdict['port']))
        self.set_setting('server.socket_host', self.validator.cdict['host'])
        self.set_setting('tools.sessions.on', True)
        self.set_setting('tools.sessions.timeout', 60)
        self.set_setting('tools.encode.on', True)
        self.set_setting('tools.encode.encoding', 'utf-8')
        self.set_setting('tools.trailing_slash.on', True)
        self.set_setting('sponge', self.validator.cdict)
        self.set_setting('sponge.root', self.fs.abspath(current_full_path))

        if 'extra' in self.validator.cdict:
            self.set_setting('sponge.extra', self.validator.cdict['extra'])

        application = self.validator.cdict['application']

        if 'boot' in application and isinstance(application['boot'], dict):
            path = application['boot']['path']
            call = application['boot']['callable']

            cloader = ClassLoader(path)
            function = getattr(cloader.get_module(), call)

            apply(function)

        template_dir = application['template-dir']
        template_path = self.fs.join(current_full_path, template_dir)
        self.set_setting('template.dir', template_path)

        image_dir = application['image-dir']
        image_path = self.fs.join(current_full_path, image_dir)
        self.set_setting('image.dir', image_path)

        adir = application['path']
        application_path = self.fs.join(current_full_path, adir)

        cloader = ClassLoader(application_path)

        meta_conf = {}
        static = application.get('static') or {}
        for media_url, media_dir in static.items():
            media_path = self.fs.join(current_full_path, media_dir)
            meta_conf[media_url] = {
                'tools.staticdir.on': True,
                'tools.staticdir.dir': media_path
            }

        classes = application.get('classes') or {}
        conf = meta_conf.copy()
        routed = False
        for classname, mountpoint in classes.items():
            try:
                cls = cloader.load(classname)
            except Exception, e:
                format_str = classname, application_path, unicode(e)
                sys.stderr.write('\nSponge could not find the class %s ' \
                                 'at %s, verify if your settings.yml ' \
                                 'is configured as well\n%s\n' % (format_str))

                raise SystemExit(1)

            fallback = lambda: cherrypy.tree.mount(root=cls(),
                                                   config=conf,
                                                   script_name=mountpoint)
            msg = '\nWARNING: The class %s has no routes\n' % cls.__name__
            if not hasattr(cls, '__routes__'):
                sys.stderr.write(msg)
                fallback()
                continue

            if not isinstance(cls.__routes__, list):
                fallback()
                continue

            routed = True
            dispatcher = cherrypy.dispatch.RoutesDispatcher()
            for k, v in cls.__routes__:
                if k is None:
                    k = "%s.%s" % (cls.__name__, v['method'])

                part1 = mountpoint.rstrip('/')
                part2 = v['route'].lstrip('/')
                new_route = "/".join([part1, part2])
                dispatcher.connect(name=k,
                                   route=new_route,
                                   controller=cls(),
                                   action=v['method'])

            conf[mountpoint] = {'request.dispatch': dispatcher}

        if routed:
            cherrypy.tree.mount(root=None, config=conf)



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
from re import escape
from mox import Mox
from nose.tools import assert_equals
from utils import assert_raises

from sponge.core import ConfigValidator, RequiredOptionError
from sponge.core import InvalidValueError

FULL_CONFIG_BASE = {
    'run-as': 'wsgi',
    'host': '0.0.0.0',
    'port': 80,
    'autoreload': False,
    'application': {
        'classes': {
            'SomeController': '/'
        },
        'image-dir': '/home/user/projects/web-app/images',
        'path': '/home/user/projects/web-app/module',
        'template-dir': '/home/user/projects/web-app/html'
    },
    'static': {
        '/media': '/home/user/projects/web-app/static',
    },
    'databases': {
        'general': 'mysql://root@localhost/general'
    }
}

def test_assert_has_inner_class_anyvalue():
    assert hasattr(ConfigValidator, 'AnyValue'), \
           'ConfigValidator should have attribute AnyValue'
    assert isinstance(ConfigValidator.AnyValue, type), \
           'ConfigValidator.AnyValue should be a class'

def test_assert_any_value_takes_type():
    assert_raises(TypeError, ConfigValidator.AnyValue, '',
                  exc_pattern=r'ConfigValidator.AnyValue takes a ' \
                  'type as parameter, got \'\'')

def test_any_value_stashes_vartype():
    av = ConfigValidator.AnyValue(bool)
    assert_equals(av.vartype, bool)

def assert_required_option(option, method, *args, **kw):
    p = r'You get to set "%s" option within settings.yml' % escape(option)
    assert_raises(RequiredOptionError, method, exc_pattern=p, *args, **kw)

def assert_invalid_option(option, value, method, *args, **kw):
    if isinstance(value, basestring):
        value = escape(value)
    else:
        value = repr(value)

    p = r'Invalid value in "%s" option: "%s". ' \
        'Read the Sponge documentation for more ' \
        'information.' % (escape(option), value)

    assert_raises(InvalidValueError, method, exc_pattern=p, *args, **kw)

def test_config_validator_takes_dict():
    assert_raises(TypeError,
                  ConfigValidator,
                  None,
                  exc_pattern=r'ConfigValidator takes a dict as ' \
                  'parameter, got None.')

def test_config_validator_has_method_validate():
    assert hasattr(ConfigValidator, 'validate'), \
           'ConfigValidator should have the method validate'
    assert callable(ConfigValidator.validate), \
           'ConfigValidator.validate should be callable'

def test_config_validator_has_method_validate_mandatory():
    assert hasattr(ConfigValidator, 'validate_mandatory'), \
           'ConfigValidator should have the method validate_mandatory'
    assert callable(ConfigValidator.validate_mandatory), \
           'ConfigValidator.validate_mandatory should be callable'

def test_config_validator_validate_calls_validation_methods():
    mocker = Mox()
    cp = ConfigValidator({})
    cp.validate_mandatory = mocker.CreateMockAnything()
    cp.validate_optional = mocker.CreateMockAnything()

    cp.validate_mandatory()

    mocker.ReplayAll()
    cp.validate()
    mocker.VerifyAll()

def test_has_method_raise_invalid():
    assert hasattr(ConfigValidator, 'raise_invalid'), \
           'ConfigValidator should have the method raise_invalid'
    assert callable(ConfigValidator.raise_invalid), \
           'ConfigValidator.raise_invalid should be callable'

def test_raise_invalid_raises():
    cp = ConfigValidator({})
    assert_invalid_option('foo-bar', 'john-doe',
                          cp.raise_invalid, 'foo-bar', 'john-doe')

def test_validate_mandatory_requires_option_run_as():
    d = FULL_CONFIG_BASE.copy()
    del d['run-as']
    cp = ConfigValidator(d)
    assert_required_option('run-as', cp.validate_mandatory)

def test_invalid_mandatory_option_run_as():
    d = FULL_CONFIG_BASE.copy()
    d['run-as'] = 'blabla'
    cp = ConfigValidator(d)
    assert_invalid_option('run-as', 'blabla', cp.validate_mandatory)

def test_validate_mandatory_option_run_as_wsgi():
    d = FULL_CONFIG_BASE.copy()
    d['run-as'] = 'wsgi'
    cp = ConfigValidator(d)
    assert cp.validate_mandatory()

def test_validate_mandatory_option_run_as_standalone():
    d = FULL_CONFIG_BASE.copy()
    d['run-as'] = 'standalone'
    cp = ConfigValidator(d)
    assert cp.validate_mandatory()

def test_validate_mandatory_requires_option_host():
    d = FULL_CONFIG_BASE.copy()
    del d['host']
    cp = ConfigValidator(d)
    assert_required_option('host', cp.validate_mandatory)

def test_invalid_mandatory_option_host():
    d = FULL_CONFIG_BASE.copy()
    d['host'] = 'invalid_host_string'
    cp = ConfigValidator(d)
    assert_invalid_option('host', 'invalid_host_string',
                          cp.validate_mandatory)

def test_validate_option_host():
    d = FULL_CONFIG_BASE.copy()
    d['host'] = '127.0.0.1'
    cp = ConfigValidator(d)
    assert cp.validate_mandatory()

def test_validate_mandatory_requires_option_port():
    d = FULL_CONFIG_BASE.copy()
    del d['port']
    cp = ConfigValidator(d)
    assert_required_option('port', cp.validate_mandatory)

def test_invalid_mandatory_option_port_float():
    d = FULL_CONFIG_BASE.copy()
    d['port'] = 90.2
    cp = ConfigValidator(d)
    assert_invalid_option('port', '90.2',
                          cp.validate_mandatory)

def test_invalid_mandatory_option_port_string():
    d = FULL_CONFIG_BASE.copy()
    d['port'] = 'invalid_port'
    cp = ConfigValidator(d)
    assert_invalid_option('port', 'invalid_port',
                          cp.validate_mandatory)

def test_validate_option_port():
    d = FULL_CONFIG_BASE.copy()
    d['port'] = '8080'
    cp = ConfigValidator(d)
    assert cp.validate_mandatory()

def test_validate_mandatory_requires_option_autoreload():
    d = FULL_CONFIG_BASE.copy()
    del d['autoreload']
    cp = ConfigValidator(d)
    assert_required_option('autoreload', cp.validate_mandatory)

def test_invalid_mandatory_option_autoreload_string():
    d = FULL_CONFIG_BASE.copy()
    d['autoreload'] = 'should_be_bool'
    cp = ConfigValidator(d)
    assert_invalid_option('autoreload', 'should_be_bool',
                          cp.validate_mandatory)

def test_validate_option_autoreload():
    d = FULL_CONFIG_BASE.copy()
    d['autoreload'] = True
    cp = ConfigValidator(d)
    assert cp.validate_mandatory()

def test_validate_mandatory_requires_option_application():
    d = FULL_CONFIG_BASE.copy()
    del d['application']
    cp = ConfigValidator(d)
    assert_required_option('application', cp.validate_mandatory)

def test_invalid_mandatory_option_application_string():
    d = FULL_CONFIG_BASE.copy()
    d['application'] = 'should_be_dict'
    cp = ConfigValidator(d)
    assert_invalid_option('application', 'should_be_dict',
                          cp.validate_mandatory)

def test_invalid_mandatory_option_application_none():
    d = FULL_CONFIG_BASE.copy()
    d['application'] = None
    cp = ConfigValidator(d)
    assert_invalid_option('application', None,
                          cp.validate_mandatory)

def test_application_invalid_controller_name_numeral():
    d = FULL_CONFIG_BASE.copy()
    d['application'] = {
        'classes':{
            '5NumeralController': '/',
        }
    }

    cp = ConfigValidator(d)
    assert_invalid_option('classes', '5NumeralController',
                          cp.validate_mandatory)

def test_application_invalid_controller_name_bad_characters():
    d = FULL_CONFIG_BASE.copy()
    d['application'] = {
        'classes':{
            '-040%$WeirdNameController': '/',
        }
    }

    cp = ConfigValidator(d)
    assert_invalid_option('classes', '-040%$WeirdNameController',
                          cp.validate_mandatory)

def test_application_invalid_controller_name_spaces():
    d = FULL_CONFIG_BASE.copy()
    d['application'] = {
        'classes':{
            'Controller With Spaces': '/',
        }
    }

    cp = ConfigValidator(d)
    assert_invalid_option('classes', 'Controller With Spaces',
                          cp.validate_mandatory)

def test_controller_url_should_start_with_slash():
    d = FULL_CONFIG_BASE.copy()
    d['application'] = {
        'classes':{
            'Controller With Spaces': 'wee/',
        }
    }

    cp = ConfigValidator(d)
    assert_invalid_option('classes', 'Controller With Spaces',
                          cp.validate_mandatory)

def test_application_invalid_names_with_spaces():
    d = FULL_CONFIG_BASE.copy()
    d['application'] = {
        'path with spaces': '/',
    }

    cp = ConfigValidator(d)
    assert_invalid_option('application', 'path with spaces',
                          cp.validate_mandatory)

def test_application_invalid_names_with_bad_characters():
    d = FULL_CONFIG_BASE.copy()
    d['application'] = {
        'fjkdas#@$%*kdfhagf': '/',
    }

    cp = ConfigValidator(d)
    assert_invalid_option('application', 'fjkdas#@$%*kdfhagf',
                          cp.validate_mandatory)

def test_application_invalid_names_starting_with_number():
    d = FULL_CONFIG_BASE.copy()
    d['application'] = {
        '3kdjfbsff': '/',
    }

    cp = ConfigValidator(d)
    assert_invalid_option('application', '3kdjfbsff',
                          cp.validate_mandatory)

def test_application_invalid_values_without_bar_at_start():
    d = FULL_CONFIG_BASE.copy()
    d['application'] = {
        'something': 'asd/',
    }

    cp = ConfigValidator(d)
    assert_invalid_option('something', 'asd/',
                          cp.validate_mandatory)

def test_validate_option_application():
    d = FULL_CONFIG_BASE.copy()
    d['application'] = {
        'image-dir': '/home/user/projects/web-app/images',
        'path': '/home/user/projects/web-app/module',
        'template-dir': '/home/user/projects/web-app/html',
        'classes':{
            'RootController': '/',
            'WikiController': '/wiki',
            'MediaController': '/media',
        }
    }
    cp = ConfigValidator(d)
    assert cp.validate_mandatory()

def test_validate_sub_options_should_be_dict():
    d = FULL_CONFIG_BASE.copy()
    d['databases'] = {
        'classes': 213
    }
    cp = ConfigValidator(d)
    assert_invalid_option('classes', '213',
                          cp.validate_mandatory)

def test_validate_mandatory_does_not_require_option_databases():
    d = FULL_CONFIG_BASE.copy()
    del d['databases']
    cp = ConfigValidator(d)
    cp.validate_mandatory()

def test_invalid_mandatory_option_databases_string():
    d = FULL_CONFIG_BASE.copy()
    d['databases'] = 'should_be_dict'
    cp = ConfigValidator(d)
    assert_invalid_option('databases', 'should_be_dict',
                          cp.validate_mandatory)

def test_invalid_mandatory_option_databases_none():
    d = FULL_CONFIG_BASE.copy()
    d['databases'] = None
    cp = ConfigValidator(d)
    assert_invalid_option('databases', None,
                          cp.validate_mandatory)

def test_databases_invalid_controller_name_weird_charactes():
    d = FULL_CONFIG_BASE.copy()
    d['databases'] = {
        '%$*': 'sqlite://asdasd',
    }

    cp = ConfigValidator(d)
    assert_invalid_option('databases', '%$*',
                          cp.validate_mandatory)

def test_validate_mandatory_does_not_requires_option_static():
    d = FULL_CONFIG_BASE.copy()
    del d['static']
    cp = ConfigValidator(d)
    cp.validate_mandatory()

def test_invalid_mandatory_option_static_int():
    d = FULL_CONFIG_BASE.copy()
    d['static'] = 10
    cp = ConfigValidator(d)
    assert_invalid_option('static', '10',
                          cp.validate_mandatory)

def test_invalid_mandatory_option_static_none():
    d = FULL_CONFIG_BASE.copy()
    d['static'] = None
    cp = ConfigValidator(d)
    assert_invalid_option('static', 'None',
                          cp.validate_mandatory)

def test_invalid_mandatory_option_static_string():
    d = FULL_CONFIG_BASE.copy()
    d['static'] = 'should_be_dict'
    cp = ConfigValidator(d)
    assert_invalid_option('static', 'should_be_dict',
                          cp.validate_mandatory)

def test_static_invalid_controller_name_weird_charactes():
    d = FULL_CONFIG_BASE.copy()
    d['static'] = {
        '%$*': 'sqlite://asdasd',
    }

    cp = ConfigValidator(d)
    assert_invalid_option('static', '%$*',
                          cp.validate_mandatory)



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
# Copyright (C) 2009 Bernardo Heynemann <heynemann@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

import sys
import yaml
import cherrypy
import optparse

from os.path import abspath, join, dirname
from mox import Mox
from nose.tools import assert_equals
from utils import assert_raises

from sponge import bob
from StringIO import StringIO

basic_config = {
    'run-as': 'wsgi',
    'host': '0.0.0.0',
    'port': 4000,
    'autoreload': True,
    'application': {
        'classes': {
            'HelloWorldController': '/',
            'AjaxController': '/ajax',
        },
        'image-dir': None,
        'path': None,
        'template-dir': None,
        'static': {
            '/media': None,
        },
    },
}

def test_can_create_bob():
    b = bob.Bob()
    assert b
    assert isinstance(b, bob.Bob)

def test_run_fails_with_unknown_args():
    mox = Mox()

    mock_parser = mox.CreateMockAnything()
    mock_parser.parse_args().AndReturn(("options", ["args"]))

    mox.ReplayAll()

    b = bob.Bob(parser=mock_parser)
    sys.stderr = StringIO()
    assert_raises(SystemExit, b.run)
    assert_equals(sys.stderr.getvalue(),
                  '\nargs is an invalid argument, choose one ' \
                  'in create, go, start\n')
    sys.stderr = sys.__stderr__
    mox.VerifyAll()

def test_run_fails_without_args():
    mox = Mox()

    mock_parser = mox.CreateMockAnything()
    mock_parser.parse_args().AndReturn(("options", []))

    mox.ReplayAll()

    b = bob.Bob(parser=mock_parser)
    sys.stderr = StringIO()
    assert_raises(SystemExit, b.run)
    assert_equals(sys.stderr.getvalue(),
                  '\nmissing argument, choose one ' \
                  'in create, go, start\n')
    sys.stderr = sys.__stderr__
    mox.VerifyAll()

def test_run_calls_create_with_second_argument():
    mox = Mox()

    mock_parser = mox.CreateMockAnything()
    mock_parser.parse_args().AndReturn(("options", ['create', 'some']))
    b = bob.Bob(parser=mock_parser)
    b.create = mox.CreateMockAnything()
    b.create('some')

    mox.ReplayAll()
    b.run()
    mox.VerifyAll()

def test_run_calls_go():
    mox = Mox()

    mock_parser = mox.CreateMockAnything()
    mock_parser.parse_args().AndReturn(("options", ['go']))
    b = bob.Bob(parser=mock_parser)
    b.go = mox.CreateMockAnything()
    b.go()

    mox.ReplayAll()
    b.run()
    mox.VerifyAll()

def test_run_calls_start_with_second_argument():
    mox = Mox()

    mock_parser = mox.CreateMockAnything()
    mock_parser.parse_args().AndReturn(("options", ['start', 'some']))
    b = bob.Bob(parser=mock_parser)
    b.start = mox.CreateMockAnything()
    b.start('some')

    mox.ReplayAll()
    b.run()
    mox.VerifyAll()

def test_go_through_main_run():
    mox = Mox()
    bobby = bob.Bob
    old_sys = bob.sys

    mock_parser = mox.CreateMockAnything()
    file_system = mox.CreateMockAnything()

    bob.sys = mox.CreateMockAnything()

    bob_mock = mox.CreateMockAnything()
    bob_instance_mock = mox.CreateMockAnything()
    bob_instance_mock.run = mox.CreateMockAnything()

    bob_instance_mock.run().AndReturn(0)
    bob_mock.__call__(parser=mock_parser, fs=file_system).AndReturn(bob_instance_mock)
    bob.sys.exit(0)
    bob.Bob = bob_mock
    mox.ReplayAll()

    try:
        got = bob.run(parser=mock_parser, fs=file_system)
        mox.VerifyAll()
    finally:
        mox.UnsetStubs()
        bob.Bob = bobby
        bob.sys = old_sys

def test_exit_without_args():
    mox = Mox()
    mock_parser = mox.CreateMockAnything()

    b = bob.Bob(parser=mock_parser)
    assert_raises(SystemExit, b.exit, exc_pattern=r'1')

def test_exit_with_specific_exit_code():
    mox = Mox()
    mock_parser = mox.CreateMockAnything()

    b = bob.Bob(parser=mock_parser)
    assert_raises(SystemExit, b.exit, 100, exc_pattern=r'100')

def test_configure():
    mox = Mox()
    mock_parser = mox.CreateMockAnything()

    mox.StubOutWithMock(bob, 'yaml')
    config_validator = bob.ConfigValidator
    sponge_config = bob.SpongeConfig

    bob.ConfigValidator = mox.CreateMockAnything()
    bob.SpongeConfig = mox.CreateMockAnything()

    b = bob.Bob(parser=mock_parser)
    b.fs = mox.CreateMockAnything()

    b.fs.current_dir().AndReturn('should_be_current_dir')
    b.fs.current_dir('settings.yml'). \
         AndReturn('/current/path/settings-yaml')

    file_mock = mox.CreateMockAnything()
    b.fs.open('/current/path/settings-yaml', 'r'). \
         AndReturn(file_mock)

    file_mock.read().AndReturn('should-be-raw-yaml-text')
    bob.yaml.load('should-be-raw-yaml-text'). \
        AndReturn('should-be-config-dict')

    bob.ConfigValidator('should-be-config-dict'). \
        AndReturn('should-be-validator')

    config_mock = mox.CreateMockAnything()
    bob.SpongeConfig(cherrypy.config, 'should-be-validator'). \
        AndReturn(config_mock)

    config_mock.setup_all('should_be_current_dir')

    mox.ReplayAll()
    try:
        b.configure()
        mox.VerifyAll()
    finally:
        mox.UnsetStubs()
        bob.ConfigValidator = config_validator
        bob.SpongeConfig = sponge_config

def test_go():
    mox = Mox()
    mox.StubOutWithMock(bob, 'cherrypy')

    b = bob.Bob()
    b.configure = mox.CreateMockAnything()
    b.configure()
    bob.cherrypy.quickstart()
    mox.ReplayAll()
    try:
        b.go()
        mox.VerifyAll()
    finally:
        mox.UnsetStubs()

def test_create_fails_without_argument():
    b = bob.Bob()
    sys.stderr = StringIO()
    assert_raises(SystemExit, b.create, None)
    assert_equals(sys.stderr.getvalue(),
                  '\nmissing project name, try something like ' \
                  '"bob create foobar"\n')
    sys.stderr = sys.__stderr__

def test_create_fails_if_path_already_exists():
    mox = Mox()
    b = bob.Bob()
    b.fs = mox.CreateMockAnything()

    b.fs.current_dir('my-project'). \
         AndReturn('/full/path/to/my-project')

    b.fs.exists('/full/path/to/my-project'). \
         AndReturn(True)

    mox.ReplayAll()
    try:
        sys.stderr = StringIO()
        assert_raises(SystemExit, b.create, 'my-project')
        assert_equals(sys.stderr.getvalue(),
                      '\nThe path "/full/path/to/my-project" ' \
                      'already exists. Maybe you could choose ' \
                      'another name for your project ?\n')
    finally:
        sys.stderr = sys.__stderr__

def test_create_success():
    mox = Mox()
    b = bob.Bob()
    b.fs = mox.CreateMockAnything()
    b.fs.join = join

    mox.StubOutWithMock(bob, 'SpongeData')
    mox.StubOutWithMock(bob, 'yaml')

    full_path = '/full/path/to/my-project'
    b.fs.current_dir('my-project'). \
         AndReturn(full_path)

    b.fs.exists(full_path). \
         AndReturn(False)

    b.fs.mkdir(full_path)

    file_mock = mox.CreateMockAnything()
    b.fs.open(join(full_path, 'settings.yml'), 'w'). \
         AndReturn(file_mock)

    expected_dict = basic_config.copy()
    expected_dict['application'].update({
        'static': {
            '/media': join('media')
        },
        'path': join('app', 'controllers.py'),
        'image-dir': join('media', 'img'),
        'template-dir': join('templates'),
    })

    bob.yaml.dump(expected_dict, indent=True).AndReturn('should-be-a-yaml')
    file_mock.write('should-be-a-yaml')
    file_mock.close()

    bob.SpongeData.get_file('project.zip'). \
        AndReturn('should-be-path-to-zip-file')

    b.fs.extract_zip('should-be-path-to-zip-file', full_path)

    mox.ReplayAll()
    b.create('my-project')
    mox.VerifyAll()

def test_start():
    mox = Mox()
    b = bob.Bob()
    b.fs = mox.CreateMockAnything()
    b.create = mox.CreateMockAnything()
    b.go = mox.CreateMockAnything()

    b.create('foo-bar')
    b.fs.pushd('foo-bar')
    b.go()

    mox.ReplayAll()

    b.start('foo-bar')

    mox.VerifyAll()

def test_fix_yml():
    expected = """
test:
  with:
    Items: here
    And: Here
"""
    wrong = """
test:
  with: {Items: here, And: Here}
"""

    b = bob.Bob()

    got = b.fix_yml(wrong)

    assert_equals(got, expected)

def test_bob_help():
    b = bob.Bob()
    assert_equals(b.get_help(), "\n Sponge Bob is the responsible for " \
                  "managing\n    the user's application and its modules. " \
                  "\n\nTo use type %prog [options] or %prog -h (--help) " \
                  "for help with the available options\n\nACTIONS:\n\ncreate " \
                  "<projectname> - creates a new project, which means " \
                  "creating a new folder in current directory, named " \
                  "projectname\ngo start the cherrypy server using the " \
                  "configuration file settings.yml in current directory." \
                  "\nstart <projectname> executes both bob create and bob go")



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
import re
import Image
import cherrypy

from mox import Mox
from utils import assert_raises
from os.path import join

from sponge.helpers import image

def test_image_has_function_jpeg():
    assert hasattr(image, 'jpeg'), 'sponge.image should have the function jpeg'
    assert callable(image.jpeg), 'sponge.image.jpeg should be callable'

def test_jpeg_takes_path_as_param():
    assert_raises(TypeError, image.jpeg, exc_pattern=r'jpeg.. takes at least 1 argument .0 given.')

def test_jpeg_param_should_be_string():
    assert_raises(TypeError, image.jpeg, None, exc_pattern=r'jpeg.. takes a string as parameter, got None.')

def test_jpeg_success():
    mox = Mox()

    path = '/path/to/mocked/img.jpg'

    mox.StubOutWithMock(image, 'Image')
    mox.StubOutWithMock(image, 'StringIO')

    stringio_mock = mox.CreateMockAnything()
    return_mock = mox.CreateMockAnything()
    img_mock = mox.CreateMockAnything()

    stringio_mock.getvalue().AndReturn(return_mock)

    image.StringIO.StringIO().AndReturn(stringio_mock)
    image.Image.open(path).AndReturn(img_mock)

    img_mock.save(stringio_mock, "JPEG", quality=100)

    cherrypy.config['image.dir'] = path

    mox.ReplayAll()

    return_got = image.jpeg(path)
    assert return_got == return_mock, 'The return of image.jpeg() should be %r, got %r' % (return_mock, return_got)
    mime = cherrypy.response.headers['Content-type']
    assert mime == 'image/jpeg', 'The response header "Content-type" should be image/jpeg, but got %r' % mime

    mox.VerifyAll()

    del cherrypy.config['image.dir']

def test_jpeg_return_string_when_file_not_found():
    filename = 'foo-file.jpg'
    path = join('bazbar', filename)
    mox = Mox()

    mox.StubOutWithMock(image, 'Image')

    image.Image.open(path).AndRaise(IOError('File not found: foo-file.jpg'))

    mox.ReplayAll()
    ret = image.jpeg(filename, base_path='bazbar')

    assert isinstance(ret, unicode), 'The return value should be unicode, but is %r' % type(ret)
    assert ret == 'File not found: foo-file.jpg', 'Wrong error description: %r' % ret

    mox.VerifyAll()

def test_crop_to_fit_bigger():
    img = Image.new('RGBA', (653, 342))
    ret = image.crop_to_fit(img, (320, 240))
    assert ret.size == (320, 240), 'Got expected size 320x240, got %rx%r.' % ret.size

def test_crop_to_fit_lower():
    img = Image.new('RGBA', (500, 750))
    ret = image.crop_to_fit(img, (320, 240))
    assert ret.size == (320, 240), 'Got expected size 320x240, got %rx%r.' % ret.size

def test_picture_takes_3_parameters():
    assert_raises(TypeError, image.picture, exc_pattern=r'picture.. takes at least 3 arguments .0 given.')

def test_picture_first_param_should_be_string():
    assert_raises(TypeError, image.picture, None, None, None, exc_pattern=r'picture.. takes a string as path parameter, got None.')

def test_picture_second_param_should_be_int():
    assert_raises(TypeError, image.picture, '', None, None, exc_pattern=r'picture.. takes a integer as width parameter, got None.')

def test_picture_third_param_should_be_int():
    assert_raises(TypeError, image.picture, '', 1, None, exc_pattern=r'picture.. takes a integer as height parameter, got None.')

def test_picture_with_crop_true_will_crop_to_fit():
    base_path = '/basepath/for/test_picture_success'
    path = 'my_picture.jpg'

    mox = Mox()

    mox.StubOutWithMock(image, 'Image')
    mox.StubOutWithMock(image, 'StringIO')
    mox.StubOutWithMock(image, 'crop_to_fit')

    img_mock = mox.CreateMockAnything()
    img_mock.size = 300, 300

    stringio_mock = mox.CreateMockAnything()
    return_mock = mox.CreateMockAnything()

    stringio_mock.getvalue().AndReturn(return_mock)

    image.StringIO.StringIO().AndReturn(stringio_mock)

    cherrypy.config['image.dir'] = base_path

    image.Image.open(join(base_path, path)).AndReturn(img_mock)
    img_mock.save(stringio_mock, 'JPEG', quality=100)
    image.crop_to_fit(img_mock, (100, 100)).AndReturn(img_mock)

    mox.ReplayAll()

    ret = image.picture(path, 100, 100, crop=True, center=False)
    assert ret == return_mock, "Expected %r. Got %r." % (return_mock, ret)

    mox.VerifyAll()
    del cherrypy.config['image.dir']

def test_picture_with_center_true_will_create_new_image_and_paste():
    base_path = '/base/path'
    path = 'image.jpg'

    mox = Mox()

    mox.StubOutWithMock(image, 'Image')
    mox.StubOutWithMock(image, 'StringIO')

    img_mock = mox.CreateMockAnything()
    img_mock.size = 300, 300

    stringio_mock = mox.CreateMockAnything()
    return_mock = mox.CreateMockAnything()

    stringio_mock.getvalue().AndReturn(return_mock)
    image.StringIO.StringIO().AndReturn(stringio_mock)

    cherrypy.config['image.dir'] = base_path

    new_img_mock = mox.CreateMockAnything()

    new_img_mock.paste(img_mock, (-100, -100))
    new_img_mock.save(stringio_mock, 'JPEG', quality=100)

    image.Image.open(join(base_path, path)).AndReturn(img_mock)
    image.Image.new('RGBA', (100, 100), 0xffffff).AndReturn(new_img_mock)

    mox.ReplayAll()
    ret = image.picture(path, 100, 100, crop=False, center=True)

    assert ret == return_mock, "Expected %r. Got %r." % (return_mock, ret)
    mox.VerifyAll()

    del cherrypy.config['image.dir']



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from nose.tools import assert_equals
from sponge import data

def test_get_file():
    old_file = data.__file__
    data.__file__ = '/path/to/file.py'

    got = data.SpongeData.get_file('blabla.jpg')

    assert_equals(got, '/path/to/blabla.jpg')
    data.__file__ = old_file



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

import re
import sys

def assert_raises(exception, callable, *args, **kwargs):
    '''
    Discussion
       assert_raises() adds two optional arguments: "exc_args"
       and "exc_pattern". "exc_args" is a tuple that is expected
       to match the .args attribute of the raised exception.
       "exc_pattern" is a compiled regular expression that the
       stringified raised exception is expected to match.

    Original url: http://code.activestate.com/recipes/307970/
    Author: Trent Mick

    Usage: assert_raises(ExceptionType, method_to_execute,
                          arguments_to_method, kwargs_to_method,
                          exc_pattern=r'^.+$')
    Please note that exc_pattern is not required, but if passed
    matches the exception message.

    Fail Conditions
    Fails on exception not raised, wrong exception type or
    invalid exception message.
    '''

    if "exc_args" in kwargs:
        exc_args = kwargs["exc_args"]
        del kwargs["exc_args"]
    else:
        exc_args = None
    if "exc_pattern" in kwargs:
        exc_pattern = kwargs["exc_pattern"]
        if isinstance(exc_pattern, basestring):
            exc_pattern = re.compile(exc_pattern)

        del kwargs["exc_pattern"]
    else:
        exc_pattern = None

    argv = [repr(a) for a in args]\
           + ["%s=%r" % (k,v)  for k,v in kwargs.items()]
    callsig = "%s(%s)" % (callable.__name__, ", ".join(argv))

    try:
        callable(*args, **kwargs)
    except exception, exc:
        if exc_args is not None:
            assert exc.args != exc_args, \
                        "%s raised %s with unexpected args: "\
                        "expected=%r, actual=%r"\
                        % (callsig, exc.__class__, exc_args, exc.args)
        if exc_pattern is not None:
            assert exc_pattern.search(str(exc)), \
                            "%s raised %s, but the exception "\
                            "does not match '%s': %r"\
                            % (callsig, exc.__class__, exc_pattern.pattern,
                               str(exc))
    except Exception, e:
        assert False, "%s raised an unexpected exception type: "\
                  "expected=%s, actual=%r"\
                  % (callsig, exception, e)
    else:
        assert False, "%s did not raise %s" % (callsig, exception)




#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
# Copyright (C) 2009 Bernardo Heynemann <heynemann@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
from sponge.bob import Bob

def test_get_file_path():
    b = Bob()
    path = b.get_file_path()

    # ignoring the "c" of "pyc", if any
    path = path.rstrip("c")
    assert path.endswith('sponge/bob/__init__.py')



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
import cherrypy

from mox import Mox
from nose.tools import assert_equal
from utils import assert_raises
from sponge.helpers import pagination

class TestPaginator:
    def check_paginator(self, params, output):
        """
        Helper method that instantiates a pagination.Paginator object from the passed
        params and then checks that its attributes match the passed output.
        """
        count, num_pages, page_range = output
        paginator = pagination.Paginator(*params)
        self.check_attribute('count', paginator, count, params)
        self.check_attribute('num_pages', paginator, num_pages, params)
        self.check_attribute('page_range', paginator, page_range, params)

    def check_attribute(self, name, paginator, expected, params):
        """
        Helper method that checks a single attribute and gives a nice error
        message upon test failure.
        """
        got = getattr(paginator, name)
        assert_equal(expected, got,
            "For '%s', expected %s but got %s.  pagination.Paginator parameters were: %s"
            % (name, expected, got, params))

    def test_paginator(self):
        """
        Tests the paginator attributes using varying inputs.
        """
        nine = [1, 2, 3, 4, 5, 6, 7, 8, 9]
        ten = nine + [10]
        eleven = ten + [11]
        tests = (
            # Each item is two tuples:
            #     First tuple is pagination.Paginator parameters - object_list, per_page,
            #         orphans, and allow_empty_first_page.
            #     Second tuple is resulting pagination.Paginator attributes - count,
            #         num_pages, and page_range.
            # Ten items, varying orphans, no empty first page.
            ((ten, 4, 0, False), (10, 3, [1, 2, 3])),
            ((ten, 4, 1, False), (10, 3, [1, 2, 3])),
            ((ten, 4, 2, False), (10, 2, [1, 2])),
            ((ten, 4, 5, False), (10, 2, [1, 2])),
            ((ten, 4, 6, False), (10, 1, [1])),
            # Ten items, varying orphans, allow empty first page.
            ((ten, 4, 0, True), (10, 3, [1, 2, 3])),
            ((ten, 4, 1, True), (10, 3, [1, 2, 3])),
            ((ten, 4, 2, True), (10, 2, [1, 2])),
            ((ten, 4, 5, True), (10, 2, [1, 2])),
            ((ten, 4, 6, True), (10, 1, [1])),
            # One item, varying orphans, no empty first page.
            (([1], 4, 0, False), (1, 1, [1])),
            (([1], 4, 1, False), (1, 1, [1])),
            (([1], 4, 2, False), (1, 1, [1])),
            # One item, varying orphans, allow empty first page.
            (([1], 4, 0, True), (1, 1, [1])),
            (([1], 4, 1, True), (1, 1, [1])),
            (([1], 4, 2, True), (1, 1, [1])),
            # Zero items, varying orphans, no empty first page.
            (([], 4, 0, False), (0, 0, [])),
            (([], 4, 1, False), (0, 0, [])),
            (([], 4, 2, False), (0, 0, [])),
            # Zero items, varying orphans, allow empty first page.
            (([], 4, 0, True), (0, 1, [1])),
            (([], 4, 1, True), (0, 1, [1])),
            (([], 4, 2, True), (0, 1, [1])),
            # Number if items one less than per_page.
            (([], 1, 0, True), (0, 1, [1])),
            (([], 1, 0, False), (0, 0, [])),
            (([1], 2, 0, True), (1, 1, [1])),
            ((nine, 10, 0, True), (9, 1, [1])),
            # Number if items equal to per_page.
            (([1], 1, 0, True), (1, 1, [1])),
            (([1, 2], 2, 0, True), (2, 1, [1])),
            ((ten, 10, 0, True), (10, 1, [1])),
            # Number if items one more than per_page.
            (([1, 2], 1, 0, True), (2, 2, [1, 2])),
            (([1, 2, 3], 2, 0, True), (3, 2, [1, 2])),
            ((eleven, 10, 0, True), (11, 2, [1, 2])),
            # Number if items one more than per_page with one orphan.
            (([1, 2], 1, 1, True), (2, 1, [1])),
            (([1, 2, 3], 2, 1, True), (3, 1, [1])),
            ((eleven, 10, 1, True), (11, 1, [1])),
        )
        for params, output in tests:
            self.check_paginator(params, output)

    def check_indexes(self, params, page_num, indexes):
        """
        Helper method that instantiates a pagination.Paginator object from the passed
        params and then checks that the start and end indexes of the passed
        page_num match those given as a 2-tuple in indexes.
        """
        paginator = pagination.Paginator(*params)
        if page_num == 'first':
            page_num = 1
        elif page_num == 'last':
            page_num = paginator.num_pages
        page = paginator.page(page_num)
        start, end = indexes
        msg = ("For %s of page %s, expected %s but got %s."
               " pagination.Paginator parameters were: %s")
        assert_equal(start, page.start_index(),
            msg % ('start index', page_num, start, page.start_index(), params))
        assert_equal(end, page.end_index(),
            msg % ('end index', page_num, end, page.end_index(), params))

    def test_page_indexes(self):
        """
        Tests that paginator pages have the correct start and end indexes.
        """
        ten = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
        tests = (
            # Each item is three tuples:
            #     First tuple is pagination.Paginator parameters - object_list, per_page,
            #         orphans, and allow_empty_first_page.
            #     Second tuple is the start and end indexes of the first page.
            #     Third tuple is the start and end indexes of the last page.
            # Ten items, varying per_page, no orphans.
            ((ten, 1, 0, True), (1, 1), (10, 10)),
            ((ten, 2, 0, True), (1, 2), (9, 10)),
            ((ten, 3, 0, True), (1, 3), (10, 10)),
            ((ten, 5, 0, True), (1, 5), (6, 10)),
            # Ten items, varying per_page, with orphans.
            ((ten, 1, 1, True), (1, 1), (9, 10)),
            ((ten, 1, 2, True), (1, 1), (8, 10)),
            ((ten, 3, 1, True), (1, 3), (7, 10)),
            ((ten, 3, 2, True), (1, 3), (7, 10)),
            ((ten, 3, 4, True), (1, 3), (4, 10)),
            ((ten, 5, 1, True), (1, 5), (6, 10)),
            ((ten, 5, 2, True), (1, 5), (6, 10)),
            ((ten, 5, 5, True), (1, 10), (1, 10)),
            # One item, varying orphans, no empty first page.
            (([1], 4, 0, False), (1, 1), (1, 1)),
            (([1], 4, 1, False), (1, 1), (1, 1)),
            (([1], 4, 2, False), (1, 1), (1, 1)),
            # One item, varying orphans, allow empty first page.
            (([1], 4, 0, True), (1, 1), (1, 1)),
            (([1], 4, 1, True), (1, 1), (1, 1)),
            (([1], 4, 2, True), (1, 1), (1, 1)),
            # Zero items, varying orphans, allow empty first page.
            (([], 4, 0, True), (0, 0), (0, 0)),
            (([], 4, 1, True), (0, 0), (0, 0)),
            (([], 4, 2, True), (0, 0), (0, 0)),
        )
        for params, first, last in tests:
            self.check_indexes(params, 'first', first)
            self.check_indexes(params, 'last', last)
        # When no items and no empty first page, we should get pagination.EmptyPage error.
        assert_raises(pagination.EmptyPage, self.check_indexes, ([], 4, 0, False), 1, None)
        assert_raises(pagination.EmptyPage, self.check_indexes, ([], 4, 1, False), 1, None)
        assert_raises(pagination.EmptyPage, self.check_indexes, ([], 4, 2, False), 1, None)

    def test_representation(self):
        class PaginatorStub:
            num_pages = 10

        page = pagination.Page([], 2, PaginatorStub)
        assert_equal('<Page 2 of 10>', repr(page))

    def test_has_next_true(self):
        class PaginatorStub:
            num_pages = 10

        page = pagination.Page([], 2, PaginatorStub)
        message = '%s should have next page, since is the page 2 of 10' % page
        assert page.has_next(), message

    def test_has_next_false(self):
        class PaginatorStub:
            num_pages = 2

        page = pagination.Page([], 2, PaginatorStub)
        message = '%s should not have next page, since is the page 2 of 2' % page
        assert not page.has_next(), message

    def test_has_previous_true(self):
        class PaginatorStub:
            num_pages = 10

        page = pagination.Page([], 2, PaginatorStub)
        message = '%s should have previous page, since is the page 2' % page
        assert page.has_previous(), message

    def test_has_previous_false(self):
        class PaginatorStub:
            num_pages = 2

        page = pagination.Page([], 1, PaginatorStub)
        message = '%s should not have previous page, since is the page 1' % page
        assert not page.has_previous(), message

    def test_has_other_pages_true_1of2(self):
        class PaginatorStub:
            num_pages = 2

        page = pagination.Page([], 1, PaginatorStub)
        message = '%s should have other pages' % page
        assert page.has_other_pages(), message

    def test_has_other_pages_true_2of2(self):
        class PaginatorStub:
            num_pages = 2

        page = pagination.Page([], 2, PaginatorStub)
        message = '%s should have other pages' % page
        assert page.has_other_pages(), message

    def test_has_other_pages_true_2of3(self):
        class PaginatorStub:
            num_pages = 3

        page = pagination.Page([], 2, PaginatorStub)
        message = '%s should have other pages' % page
        assert page.has_other_pages(), message

    def test_has_other_pages_true_3of3(self):
        class PaginatorStub:
            num_pages = 3

        page = pagination.Page([], 3, PaginatorStub)
        message = '%s should have other pages' % page
        assert page.has_other_pages(), message

    def test_has_other_pages_false(self):
        class PaginatorStub:
            num_pages = 1

        page = pagination.Page([], 1, PaginatorStub)
        message = '%s should not have other pages' % page
        assert not page.has_other_pages(), message

    def test_next_page_number(self):
        class PaginatorStub:
            num_pages = 2

        page = pagination.Page([], 1, PaginatorStub)

        got = page.next_page_number()
        expected = 2

        message = "%s's next page number should be 2, got %r" % (page, got)
        assert got == expected, message

    def test_previous_page_number(self):
        class PaginatorStub:
            num_pages = 2

        page = pagination.Page([], 2, PaginatorStub)

        got = page.previous_page_number()
        expected = 1

        message = "%s's previous page number should be 1, got %r" % (page, got)
        assert got == expected, message

    def test_page_validate_number_not_integer(self):
        p = pagination.Paginator(range(5), 2)
        assert_raises(pagination.PageNotAnInteger,
                      p.validate_number,
                      '1sda0')

    def test_page_validate_number_less_than_1(self):
        p = pagination.Paginator(range(5), 2)
        assert_raises(pagination.EmptyPage,
                      p.validate_number,
                      -1)

    def test_page_validate_number_when_not_allowing_1st_empty_page(self):
        p = pagination.Paginator([], 2,
                                 allow_empty_first_page=False)
        assert_raises(pagination.EmptyPage,
                      p.validate_number,
                      1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# <Sponge - Lightweight Web Framework>
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
# Copyright (C) 2009 Bernardo Heynemann <heynemann@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import Image
import ImageDraw
import cherrypy
import StringIO

def jpeg(path, base_path=None):
    if not isinstance(path, basestring):
        raise TypeError('jpeg() takes a string as parameter, got %r.' % path)

    if not base_path:
        base_path = cherrypy.config['image.dir']

    fullpath = os.path.join(base_path, path)

    try:
        img = Image.open(fullpath)
    except IOError, e:
        cherrypy.response.status = 404
        return unicode(e)


    sfile = StringIO.StringIO()
    img.save(sfile, "JPEG", quality=100)
    cherrypy.response.headers['Content-type'] = "image/jpeg"
    return sfile.getvalue()

# Based in the original, public license, version from Kevin Cazabon
# <http://www.cazabon.com/python/>

def crop_to_fit(img, output_size):
    live_area = (0, 0, img.size[0] - 1, img.size[1] - 1)
    live_size = (live_area[2] - live_area[0], live_area[3] - live_area[1])

    # calculate the aspect ratio of the live_area
    live_area_aspect_ratio = float(live_size[0])/float(live_size[1])

    # calculate the aspect ratio of the output image
    aspect_ratio = float(output_size[0])/float(output_size[1])

    # figure out if the sides or top/bottom will be cropped off
    if live_area_aspect_ratio >= aspect_ratio:
        # live_area is wider than what's needed, crop the sides
        crop_width = int((aspect_ratio * float(live_size[1])) + 0.5)
        crop_height = live_size[1]
    else:
        #live_area is taller than what's needed, crop the top and bottom
        crop_width = live_size[0]
        crop_height = int((float(live_size[0]) / aspect_ratio) + 0.5)

    # make the crop
    left_side = int(live_area[0] + (float(live_size[0] - crop_width) * 0.5))
    top_side = int(live_area[1] + (float(live_size[1] - crop_height) * 0.5))


    outputImage = img.crop((left_side, top_side, left_side + crop_width, top_side + crop_height))

    # resize the image and return it
    return outputImage.resize(output_size, 3)

def picture(path,
            width,
            height,
            crop=True,
            center=True,
            background=0xffffff,
            base_path=None):

    if not isinstance(path, basestring):
        raise TypeError('picture() takes a string as path parameter, got %r.' % path)

    if not isinstance(width, int):
        raise TypeError('picture() takes a integer as width parameter, got %r.' % width)

    if not isinstance(height, int):
        raise TypeError('picture() takes a integer as height parameter, got %r.' % height)

    if not base_path:
        base_path = cherrypy.config['image.dir']

    wished_size = width, height
    img = Image.open(os.path.join(base_path, path))

    if crop:
        img = crop_to_fit(img, (width, height))

    if center:
        old_img = img
        img = Image.new('RGBA', (width, height), background)
        ow, oh = old_img.size
        left = (width - ow) / 2
        top = (height - oh) / 2
        img.paste(old_img, (left, top))

    sfile = StringIO.StringIO()
    img.save(sfile, "JPEG", quality=100)
    cherrypy.response.headers['Content-type'] = "image/jpeg"
    return sfile.getvalue()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# <Sponge - Lightweight Web Framework>
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
# Copyright (C) 2009 Bernardo Heynemann <heynemann@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import cherrypy

def route(route, name=None):
    def dec(func):
        conf = (
            name, {
                'route': route,
                'method': func.__name__
            }
        )
        return func, conf

    return dec

class MetaController(type):
    def __init__(cls, name, bases, attrs):
        if name not in ('MetaController', 'Controller'):
            cls.__routes__ = []
            for attr, value in attrs.items():
                if isinstance(value, tuple) and len(value) is 2:
                    method, conf = value
                    setattr(cls, attr, method)
                    cls.__routes__.append(conf)

        super(MetaController, cls).__init__(name, bases, attrs)

class Controller(object):
    __metaclass__ = MetaController
    __routes__ = None



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
import cherrypy

from mox import Mox
from nose.tools import assert_equal
from utils import assert_raises
from sponge.contrib import controllers

class TestImageHandler:
    def __init__(self):
        self.handler = controllers.ImageHandler()

    def test_is_exposed(self):
        message = 'The ImageHandler should be exposed, ' \
                  'so that CherryPy can see it'
        assert self.handler.exposed is True, message

    def test_return_not_found_with_no_args(self):
        got = self.handler()
        msg1 = 'the response should be "not found", got %s' % repr(got)
        msg2 = 'The response status code should be 404, got %r' % \
               cherrypy.response.status
        assert got == 'not found', msg1
        assert cherrypy.response.status == 404, msg2

    def test_calls_jpeg_when_args_length_less_than_3(self):
        mox = Mox()

        mox.StubOutWithMock(controllers, 'jpeg')
        ret = 'should_be_a_pil_img'
        controllers.jpeg(path='arg1/arg2').AndReturn(ret)

        mox.ReplayAll()
        got = self.handler('arg1', 'arg2')
        mox.VerifyAll()

        msg = 'Expected "%s", got %r' % (ret, got)
        assert got == ret, msg

    def test_calls_jpeg_when_args_length_equals_3(self):
        mox = Mox()

        mox.StubOutWithMock(controllers, 'jpeg')
        ret = 'should_be_a_pil_img'
        controllers.jpeg(path='arg1/arg2/arg3').AndReturn(ret)

        mox.ReplayAll()
        got = self.handler('arg1', 'arg2', 'arg3')
        mox.VerifyAll()

        msg = 'Expected "%s", got %r' % (ret, got)
        assert got == ret, msg

    def test_calls_jpeg_when_args_length_bigger_than_3_no_crop(self):
        mox = Mox()

        mox.StubOutWithMock(controllers, 'jpeg')
        ret = 'should_be_a_pil_img'
        controllers.jpeg(path='arg1/arg2/arg3/arg4').AndReturn(ret)

        mox.ReplayAll()
        got = self.handler('arg1', 'arg2', 'arg3', 'arg4')
        mox.VerifyAll()

        msg = 'Expected "%s", got %r' % (ret, got)
        assert got == ret, msg

    def test_calls_jpeg_when_args_length_bigger_than_3_crop_no_proportion(self):
        mox = Mox()

        mox.StubOutWithMock(controllers, 'jpeg')
        ret = 'should_be_a_pil_img'
        controllers.jpeg(path='crop/arg2/arg3/arg4').AndReturn(ret)

        mox.ReplayAll()
        got = self.handler('crop', 'arg2', 'arg3', 'arg4')
        mox.VerifyAll()

        msg = 'Expected "%s", got %r' % (ret, got)
        assert got == ret, msg

    def test_calls_jpeg_when_args_length_equals_3_and_crop(self):
        mox = Mox()

        mox.StubOutWithMock(controllers, 'jpeg')
        mox.StubOutWithMock(controllers, 'picture')

        ret = 'should_be_a_pil_img'

        controllers.jpeg(path='crop/200x100/image.jpg')
        controllers.picture(path='image.jpg', width=200, height=100). \
                                             AndReturn(ret)

        mox.ReplayAll()
        got = self.handler('crop', '200x100', 'image.jpg')
        mox.VerifyAll()

        msg = 'Expected "%s", got %r' % (ret, got)
        assert got == ret, msg

    def test_creation_takes_optional_cache_path_string(self):
        assert_raises(TypeError, controllers.ImageHandler, 5,
                      exc_pattern=r'The path given to ImageHandler ' \
                      'to cache must be a string, got 5')

    def test_caching_fails_if_cache_path_does_not_exist(self):
        mox = Mox()

        class ImageHandlerStub(controllers.ImageHandler):
            fs = mox.CreateMockAnything()

        ImageHandlerStub.fs.exists('/full/path/to/cache').AndReturn(False)

        mox.ReplayAll()
        assert_raises(controllers.InvalidCachePath,
                      ImageHandlerStub,
                      '/full/path/to/cache',
                      exc_pattern=r'The given path \(/full/path/to/cache\) ' \
                      'does not exist, so that ImageHandler can not save ' \
                      'cache files there.')
        mox.VerifyAll()

    def test_caching_return_if_already_exists(self):
        mox = Mox()

        old_jpeg = controllers.jpeg
        old_picture = controllers.picture
        old_static = controllers.static

        mox.StubOutWithMock(controllers, 'static')
        controllers.jpeg = mox.CreateMockAnything()
        controllers.picture = mox.CreateMockAnything()

        cache_at = '/full/path/to/cache'
        class ImageHandlerStub(controllers.ImageHandler):
            fs = mox.CreateMockAnything()

        ImageHandlerStub.fs.exists(cache_at).AndReturn(True)

        controllers.jpeg(path='imgs/image.jpg')
        ImageHandlerStub.fs.join(cache_at, 'imgs/image.jpg'). \
                         AndReturn('/should/be/cache/full/path.jpg')

        ImageHandlerStub.fs.exists('/should/be/cache/full/path.jpg'). \
                         AndReturn(True)
        controllers.static.serve_file('/should/be/cache/full/path.jpg',
                              'image/jpeg'). \
                   AndReturn('should-be-image-data')

        mox.ReplayAll()
        try:
            img = ImageHandlerStub(cache_at)
            assert img.should_cache
            assert_equal(img.cache_path, cache_at)
            got = img('imgs', 'image.jpg')
            assert_equal(got, 'should-be-image-data')
            mox.VerifyAll()
        finally:
            controllers.jpeg = old_jpeg
            controllers.picture = old_picture
            mox.UnsetStubs()

    def test_caching_opens_if_does_not_exist(self):
        mox = Mox()

        old_jpeg = controllers.jpeg
        old_picture = controllers.picture
        controllers.jpeg = mox.CreateMockAnything()
        controllers.picture = mox.CreateMockAnything()

        cache_at = '/full/path/to/cache'
        class ImageHandlerStub(controllers.ImageHandler):
            fs = mox.CreateMockAnything()

        ImageHandlerStub.fs.exists(cache_at).AndReturn(True)

        controllers.jpeg(path='imgs/image.jpg').AndReturn('fake-img')
        ImageHandlerStub.fs.join(cache_at, 'imgs/image.jpg'). \
                         AndReturn('/should/be/cache/full/path.jpg')

        ImageHandlerStub.fs.exists('/should/be/cache/full/path.jpg'). \
                         AndReturn(False)

        ImageHandlerStub.fs.dirname('/should/be/cache/full/path.jpg'). \
                         AndReturn('dir-name')

        ImageHandlerStub.fs.mkdir('dir-name')

        file_mock = mox.CreateMockAnything()
        ImageHandlerStub.fs.open_raw('/should/be/cache/full/path.jpg', 'w'). \
                         AndReturn(file_mock)

        file_mock.write('fake-img')
        file_mock.close()

        mox.ReplayAll()
        try:
            img = ImageHandlerStub(cache_at)
            assert img.should_cache
            assert_equal(img.cache_path, cache_at)
            got = img('imgs', 'image.jpg')
            assert_equal(got, 'fake-img')
            mox.VerifyAll()
        finally:
            controllers.jpeg = old_jpeg
            controllers.picture = old_picture




#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
from os.path import split, join
from nose.tools import assert_equals
from sponge.core.io import ClassLoader, FileSystem

def test_load_module():
    this_path = split(__file__)[0]
    relative_path = join(this_path, 'module', 'to', 'load')
    module_path = FileSystem.current_dir(relative_path)
    ClassToLoad = ClassLoader(module_path).load('ClassInsideModule')

    assert_equals(ClassToLoad.__name__, 'ClassInsideModule')
    assert hasattr(ClassToLoad, 'param'), \
           '%r should have the attribute "param"' % ClassToLoad
    assert_equals(ClassToLoad.param, 'ParamFromClass')

def test_load_file():
    this_path = split(__file__)[0]
    relative_path = join(this_path, 'module', 'to', 'load', 'some_file.py')
    module_path = FileSystem.current_dir(relative_path)
    ClassToLoad = ClassLoader(module_path).load('ClassInsideFile')

    assert_equals(ClassToLoad.__name__, 'ClassInsideFile')
    assert hasattr(ClassToLoad, 'param'), \
           '%r should have the attribute "param"' % ClassToLoad
    assert_equals(ClassToLoad.param, 'ParamFromClass')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# <Sponge - Lightweight Web Framework>
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
# Copyright (C) 2009 Bernardo Heynemann <heynemann@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import cherrypy
from genshi.template import TemplateLoader

def make_url(url):
    if not isinstance(url, basestring):
        raise TypeError('sponge.template.make_url ' \
                        'takes a string as param, got %r.' % url)
    if url.startswith('/'):
        url = url[1:]

    base = cherrypy.request.base
    if base.endswith('/'):
        base = base[:-1]

    return "%s/%s" % (base, url)

def render_html(filename, context=None, template_path=None):
    if context is None:
        context = {}

    if not isinstance(filename, basestring):
        raise TypeError('sponge.template.render_html ' \
                        'takes a string as filename param, got %r.' % filename)

    if not len(filename):
        raise TypeError('sponge.template.render_html ' \
                        'filename param can not be empty.')

    if not isinstance(context, dict):
        raise TypeError('sponge.template.render_html ' \
                        'takes a dict as context param, got %r.' % context)

    if 'make_url' in context.keys():
        msg = 'The key "make_url" is already in ' \
              'template context as: %r' % make_url
        raise KeyError(msg)

    if template_path is None:
        try:
            template_path = cherrypy.config['template.dir']
        except KeyError:
            raise LookupError('You must configure "template.dir" string in ' \
                              'CherryPy or pass template_path param to render_html')

    elif not isinstance(template_path, basestring):
        raise TypeError('sponge.template.render_html ' \
                        'takes a string as template_path param, got %r.' % template_path)

    context['make_url'] = make_url
    loader = TemplateLoader(template_path,
                            auto_reload=True)
    template = loader.load(filename)
    generator = template.generate(**context)
    return generator.render('html', doctype='html')




#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
import re
import Image
import cherrypy

from mox import Mox
from utils import assert_raises
from os.path import join

from sponge import template

def test_template_has_make_url_function():
    assert hasattr(template, 'make_url'), 'sponge.template should have the function make_url'
    assert callable(template.make_url), 'sponge.template.make_url should be callable'

def test_make_url_takes_string_as_param():
    expected = r'sponge.template.make_url ' \
               'takes a string as param, got None.'
    assert_raises(TypeError, template.make_url, None, exc_pattern=expected)

def test_make_url_without_trailling_slash():
    base_url = 'http://my.unit.test/for/ma-cherie'
    cherrypy.request.base = base_url

    expected_url = 'http://my.unit.test/for/ma-cherie/index'
    got_url = template.make_url('index')
    assert got_url == expected_url, 'Expected %s, got %s' % (expected_url, got_url)

def test_make_url_with_trailling_slash_on_base_url():
    base_url = 'http://my.unit.test/for/ma-cherie/'
    cherrypy.request.base = base_url

    expected_url = 'http://my.unit.test/for/ma-cherie/index'
    got_url = template.make_url('index')
    assert got_url == expected_url, 'Expected %s, got %s' % (expected_url, got_url)

def test_make_url_with_trailling_slash_on_url_part():
    base_url = 'http://my.unit.test/for/ma-cherie'
    cherrypy.request.base = base_url

    expected_url = 'http://my.unit.test/for/ma-cherie/index'
    got_url = template.make_url('/index')
    assert got_url == expected_url, 'Expected %s, got %s' % (expected_url, got_url)

def test_make_url_with_trailling_slash_on_both():
    base_url = 'http://my.unit.test/for/ma-cherie/'
    cherrypy.request.base = base_url

    expected_url = 'http://my.unit.test/for/ma-cherie/index'
    got_url = template.make_url('/index')
    assert got_url == expected_url, 'Expected %s, got %s' % (expected_url, got_url)

def test_template_has_function_render_html():
    assert hasattr(template, 'render_html'), 'sponge.template should have the function render_html'
    assert callable(template.render_html), 'sponge.template.render_html should be callable'

def test_templates_render_html_raises_filename_nonstring():
    assert_raises(TypeError,
                  template.render_html,
                  None,
                  {},
                  exc_pattern=r'sponge.template.render_html ' \
                  'takes a string as filename param, got None.')
    assert_raises(TypeError,
                  template.render_html,
                  5,
                  {},
                  exc_pattern=r'sponge.template.render_html ' \
                  'takes a string as filename param, got 5.')

def test_templates_render_html_raises_filename_empty():
    assert_raises(TypeError,
                  template.render_html,
                  '',
                  {},
                  exc_pattern=r'sponge.template.render_html ' \
                  'filename param can not be empty.')

def test_templates_render_html_raises_context_nondict():
    assert_raises(TypeError,
                  template.render_html,
                  'index.html',
                  'a string',
                  exc_pattern=r'sponge.template.render_html ' \
                  'takes a dict as context param, got \'a string\'.')
    assert_raises(TypeError,
                  template.render_html,
                  'index.html',
                  5,
                  exc_pattern=r'sponge.template.render_html ' \
                  'takes a dict as context param, got 5.')

def test_templates_render_html_raises_context_already_have_make_url():
    assert_raises(KeyError,
                  template.render_html,
                  'index.html',
                  {'make_url': "ss"},
                  exc_pattern=r'The key "make_url" is already in ' \
                  'template context as[:] %s' % re.escape(repr(template.make_url)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# <Sponge - Lightweight Web Framework>
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
# Copyright (C) 2009 Bernardo Heynemann <heynemann@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import sys
import yaml
import cherrypy
import optparse

from sponge import __version__ as version
from sponge.core import ConfigValidator, SpongeConfig
from sponge.core.io import FileSystem
from sponge.data import SpongeData

basic_config = {
    'run-as': 'wsgi',
    'host': '0.0.0.0',
    'port': 4000,
    'autoreload': True,
    'application': {
        'classes': {
            'HelloWorldController': '/',
            'AjaxController': '/ajax',
        },
        'image-dir': None,
        'path': None,
        'template-dir': None,
        'static': {
            '/media': None,
        },
    },
}

class Bob(object):
    """Sponge Bob is the responsible for managing
    the user's application and its modules."""

    ACTIONS = [
        ('create', '<projectname> - creates a new project, which means creating a new folder in current directory, named projectname'),
        ('go', 'start the cherrypy server using the configuration file settings.yml in current directory.'),
        ('start', '<projectname> executes both bob create and bob go'),
    ]

    def __init__(self, parser=None, fs=None):
        self.parser = parser
        if not self.parser:
            self.parser = optparse.OptionParser(usage=self.get_help(),
                                                description=__doc__,
                                                version=version)
        self.fs = fs
        if not self.fs:
            self.fs = FileSystem()

    def get_help(self):
        actions = "\n".join(["%s %s" % (k, v) for k, v in self.ACTIONS])
        usage = "\n %s \n\nTo use type %%prog [options]" \
                " or %%prog -h (--help) for help with" \
                " the available options\n\nACTIONS:\n\n%s" % (self.__doc__, actions)

        return usage

    def exit(self, code=1):
        raise SystemExit(code)

    def run(self):
        accepted = [a[0] for a in self.ACTIONS]
        options, args = self.parser.parse_args()
        error_msg = '\nBob got a error when %s.\n    %s\n'

        if not args:
            msg = '\nmissing argument, choose one in %s\n'
            sys.stderr.write(msg % ", ".join(accepted))
            self.exit()

        if args[0] not in accepted:
            msg = '\n%s is an invalid argument, choose one in %s\n'
            sys.stderr.write(msg % (args[0], ", ".join(accepted)))
            self.exit()

        return getattr(self, args[0])(*args[1:])

    def configure(self):
        current_full_path = self.fs.current_dir()

        full_path = self.fs.current_dir("settings.yml")
        raw_yaml = self.fs.open(full_path, 'r').read()
        orig_dict = yaml.load(raw_yaml)

        validator = ConfigValidator(orig_dict)
        config = SpongeConfig(cherrypy.config, validator)
        config.setup_all(current_full_path)

    def go(self):
        self.configure()
        cherrypy.quickstart()

    def create(self, project_name=None):
        if not project_name:
            error_msg = 'missing project name, try ' \
                        'something like "bob create foobar"'
            sys.stderr.write("\n%s\n" % error_msg)
            self.exit()

        path = self.fs.current_dir(project_name)

        if self.fs.exists(path):
            error_msg = 'The path "%s" already exists. ' \
                        'Maybe you could choose another ' \
                        'name for your project ?' % path

            sys.stderr.write("\n%s\n" % error_msg)
            self.exit()

        self.fs.mkdir(path)

        cfg = self.fs.open(self.fs.join(path, 'settings.yml'), 'w')
        cdict = basic_config.copy()

        media_path = self.fs.join('media')
        cdict['application']['static']['/media'] = media_path

        controller_path = self.fs.join('app', 'controllers.py')
        cdict['application']['path'] = controller_path

        image_path = self.fs.join('media', 'img')
        cdict['application']['image-dir'] = image_path

        template_path = self.fs.join('templates')
        cdict['application']['template-dir'] = template_path

        yml_data = self.fix_yml(yaml.dump(cdict, indent=True))
        cfg.write(yml_data)
        cfg.close()

        zip_file = SpongeData.get_file('project.zip')
        self.fs.extract_zip(zip_file, path)

    def fix_yml(self, yml):
        yml = re.sub('[ ]+{', '{', yml)
        pattern1 = r'(?P<spc>[ ]+)(?P<pre>[^{]+)[{](?P<cnt>[^}]+)[}]'
        replacement1 = r'\g<spc>\g<pre>\n  \g<spc>\g<cnt>'
        yml = re.sub(pattern1, replacement1, yml)
        pattern2 = r'(?P<spc>[ ]+)(?P<pre>[^,]+)[,][ ]*'
        replacement2 = r'\g<spc>\g<pre>\n  \g<spc>'
        yml = re.sub(pattern2, replacement2, yml)
        return yml

    def start(self, project_name=None):
        self.create(project_name)
        self.fs.pushd(project_name)
        self.go()

    def get_file_path(self):
        return __file__

def run(*args, **kw):
    bob = Bob(*args, **kw)
    sys.exit(bob.run())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# <Sponge - Lightweight Web Framework>
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
# Copyright (C) 2009 Bernardo Heynemann <heynemann@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import cherrypy

from cherrypy.lib import static

from sponge.core.io import FileSystem
from sponge.helpers.image import jpeg, picture

class InvalidCachePath(IOError):
    pass

class ImageHandler(object):
    exposed = True
    should_cache = False
    cache_path = None
    fs = FileSystem()

    def __init__(self, cache_at=None):
        if not isinstance(cache_at, (basestring, type(None))):
            raise TypeError, 'The path given to ImageHandler ' \
                  'to cache must be a string, got %s' % repr(cache_at)

        if cache_at:
            self.should_cache =True
            self.cache_path = cache_at

            if not self.fs.exists(cache_at):
                raise InvalidCachePath, \
                      'The given path (%s) does not exist, ' \
                      'so that ImageHandler can not save ' \
                      'cache files there.' % cache_at

    def get_cache_path(self, path):
        return self.fs.join(self.cache_path, path.lstrip('/'))

    def __call__(self, *args, **kw):
        if len(args) < 1:
            cherrypy.response.status = 404
            return "not found"

        path = "/".join(args)

        image = jpeg(path=path)

        cache_full_path = None

        if self.should_cache:
            cache_full_path = self.get_cache_path(path)
            if self.fs.exists(cache_full_path):
                return static.serve_file(cache_full_path, 'image/jpeg')

        if len(args) >= 3 and args[0] == 'crop':
            proportion = re.match(r'(?P<width>\d+)x(?P<height>\d+)',
                                  args[1])
            if proportion:
                width = int(proportion.group('width'))
                height = int(proportion.group('height'))

                image = picture(path="/".join(args[2:]),
                                width=width,
                                height=height)

        if self.should_cache:
            dir_path = self.fs.dirname(cache_full_path)
            self.fs.mkdir(dir_path)
            img_file = self.fs.open_raw(cache_full_path, 'w')
            img_file.write(image)
            img_file.close()

        return image



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
import cherrypy

from mox import Mox
from nose.tools import assert_equal
from utils import assert_raises
from sponge import controller

def test_route_return_tuple():
    @controller.route('route_name', 'route/:foo/other/:bar')
    def my_func():
        return 'it is my func'

    assert isinstance(my_func, tuple), 'A function decorated with ' \
           'route should be a tuple'
    assert len(my_func) == 2, 'the tuple should have two items'
    assert callable(my_func[0]), 'first item of tuple should be the actual callable'
    assert isinstance(my_func[1], tuple)

def test_controller_with_route_named():
    class MyController(controller.Controller):
        @controller.route('/path/:to/:route/:param3', name='my_route_name')
        def some_method(self, to, route, param3):
            return 'something'

    assert hasattr(MyController, '__routes__'), 'MyController should have __routes__'
    assert_equal(MyController.__routes__, [
        ('my_route_name', {
            'route': '/path/:to/:route/:param3',
            'method': 'some_method'
        }),
    ])

def test_controller_with_route_without_name():
    class MyController(controller.Controller):
        @controller.route('/path/:to/:route/:param3')
        def some_method(self, to, route, param3):
            return 'something'

    assert hasattr(MyController, '__routes__'), 'MyController should have __routes__'
    assert_equal(MyController.__routes__, [
        (None, {
            'route': '/path/:to/:route/:param3',
            'method': 'some_method'
        }),
    ])



# -*- coding: utf-8 -*-
import sys
from StringIO import StringIO
from mox import Mox
from nose.tools import assert_equals
from utils import assert_raises
from sponge.core import io

def test_has_a_stack_list():
    assert hasattr(io.FileSystem, 'stack'), \
           'FileSystem should have a stack'
    assert isinstance(io.FileSystem.stack, list), \
           'FileSystem.stack should be a list'

def test_instance_stack_is_not_the_same_as_class_level():
    class MyFs(io.FileSystem):
        pass

    MyFs.stack.append('foo')
    MyFs.stack.append('bar')
    assert_equals(MyFs().stack, [])

def test_pushd_appends_current_dir_to_stack_if_empty():
    mox = Mox()
    old_os = io.os
    io.os = mox.CreateMockAnything()

    class MyFs(io.FileSystem):
        stack = []

        @classmethod
        def current_dir(cls):
            return 'should be current dir'

    io.os.chdir('somewhere')

    mox.ReplayAll()
    try:
        assert len(MyFs.stack) is 0
        MyFs.pushd('somewhere')
        assert len(MyFs.stack) is 2
        assert_equals(MyFs.stack, ['should be current dir',
                                   'somewhere'])
        mox.VerifyAll()
    finally:
        io.os = old_os

def test_pushd():
    mox = Mox()
    old_os = io.os
    io.os = mox.CreateMockAnything()

    class MyFs(io.FileSystem):
        stack = ['first']

    io.os.chdir('second')

    mox.ReplayAll()
    try:
        assert len(MyFs.stack) is 1
        MyFs.pushd('second')
        assert len(MyFs.stack) is 2
        assert_equals(MyFs.stack, ['first',
                                   'second'])
        mox.VerifyAll()
    finally:
        io.os = old_os

def test_pop_with_more_than_1_item():
    mox = Mox()
    old_os = io.os
    io.os = mox.CreateMockAnything()

    class MyFs(io.FileSystem):
        stack = ['one', 'two']

    io.os.chdir('one')

    mox.ReplayAll()
    try:
        assert len(MyFs.stack) is 2
        MyFs.popd()
        assert len(MyFs.stack) is 1
        assert_equals(MyFs.stack, ['one'])
        mox.VerifyAll()
    finally:
        io.os = old_os

def test_pop_with_1_item():
    mox = Mox()
    old_os = io.os
    io.os = mox.CreateMockAnything()

    class MyFs(io.FileSystem):
        stack = ['one']

    mox.ReplayAll()
    try:
        assert len(MyFs.stack) is 1
        MyFs.popd()
        assert len(MyFs.stack) is 0
        assert_equals(MyFs.stack, [])
        mox.VerifyAll()
    finally:
        io.os = old_os

def test_pop_with_no_item():
    mox = Mox()
    old_os = io.os
    io.os = mox.CreateMockAnything()

    class MyFs(io.FileSystem):
        stack = []

    mox.ReplayAll()
    try:
        assert len(MyFs.stack) is 0
        MyFs.popd()
        assert len(MyFs.stack) is 0
        assert_equals(MyFs.stack, [])
        mox.VerifyAll()
    finally:
        io.os = old_os

def test_filename_with_extension():
    got = io.FileSystem.filename('/path/to/filename.jpg')
    assert_equals(got, 'filename.jpg')

def test_filename_without_extension():
    got = io.FileSystem.filename('/path/to/filename.jpg', False)
    assert_equals(got, 'filename')

def test_dirname():
    got = io.FileSystem.dirname('/path/to/filename.jpg')
    assert_equals(got, '/path/to')

def test_exists():
    mox = Mox()
    old_exists = io.exists
    io.exists = mox.CreateMockAnything()

    io.exists('some path').AndReturn('should be bool')

    mox.ReplayAll()
    try:
        got = io.FileSystem.exists('some path')
        assert_equals(got, 'should be bool')
        mox.VerifyAll()
    finally:
        io.exists = old_exists

def test_extract_zip_non_verbose():
    mox = Mox()
    class MyFs(io.FileSystem):
        stack = []
        abspath = mox.CreateMockAnything()
        pushd = mox.CreateMockAnything()
        popd = mox.CreateMockAnything()
        open_raw = mox.CreateMockAnything()
        mkdir = mox.CreateMockAnything()

    mox.StubOutWithMock(io, 'zipfile')

    filename = 'modafoca.zip'
    base_path = '../to/project'
    full_path = '/full/path/to/project'

    MyFs.abspath(base_path).AndReturn(full_path)
    MyFs.pushd(full_path)

    zip_mock = mox.CreateMockAnything()

    io.zipfile.ZipFile(filename).AndReturn(zip_mock)

    file_list = [
        'settings.yml',
        'app',
        'app/controllers.py'
    ]
    zip_mock.namelist().AndReturn(file_list)
    zip_mock.read('settings.yml').AndReturn('settings.yml content')
    zip_mock.read('app/controllers.py').AndReturn('controllers.py content')

    file_mock1 = mox.CreateMockAnything()
    MyFs.open_raw('settings.yml', 'w').AndReturn(file_mock1)
    file_mock1.write('settings.yml content')
    file_mock1.close()

    MyFs.open_raw('app', 'w').AndRaise(IOError('it is a directory, dumb ass!'))
    MyFs.mkdir('app')

    file_mock2 = mox.CreateMockAnything()
    MyFs.open_raw('app/controllers.py', 'w').AndReturn(file_mock2)
    file_mock2.write('controllers.py content')
    file_mock2.close()

    MyFs.popd()

    mox.ReplayAll()
    try:
        MyFs.extract_zip('modafoca.zip', base_path)
        mox.VerifyAll()
    finally:
        mox.UnsetStubs()

def test_extract_zip_verbose():
    mox = Mox()
    sys.stdout = StringIO()
    class MyFs(io.FileSystem):
        stack = []
        abspath = mox.CreateMockAnything()
        pushd = mox.CreateMockAnything()
        popd = mox.CreateMockAnything()
        open_raw = mox.CreateMockAnything()
        mkdir = mox.CreateMockAnything()

    mox.StubOutWithMock(io, 'zipfile')

    filename = 'modafoca.zip'
    base_path = '../to/project'
    full_path = '/full/path/to/project'

    MyFs.abspath(base_path).AndReturn(full_path)
    MyFs.pushd(full_path)

    zip_mock = mox.CreateMockAnything()

    io.zipfile.ZipFile(filename).AndReturn(zip_mock)

    file_list = [
        'settings.yml',
        'app',
        'app/controllers.py'
    ]
    zip_mock.namelist().AndReturn(file_list)
    zip_mock.read('settings.yml').AndReturn('settings.yml content')
    zip_mock.read('app/controllers.py').AndReturn('controllers.py content')

    file_mock1 = mox.CreateMockAnything()
    MyFs.open_raw('settings.yml', 'w').AndReturn(file_mock1)
    file_mock1.write('settings.yml content')
    file_mock1.close()

    MyFs.open_raw('app', 'w').AndRaise(IOError('it is a directory, dumb ass!'))
    MyFs.mkdir('app')

    file_mock2 = mox.CreateMockAnything()
    MyFs.open_raw('app/controllers.py', 'w').AndReturn(file_mock2)
    file_mock2.write('controllers.py content')
    file_mock2.close()

    MyFs.popd()

    mox.ReplayAll()
    try:
        MyFs.extract_zip('modafoca.zip', base_path, verbose=True)
        assert_equals(sys.stdout.getvalue(),
                      'Extracting files to /full/path/to/project\n  ' \
                      '-> Unpacking settings.yml\n  -> Unpacking app' \
                      '\n---> Creating directory app\n  -> Unpacking' \
                      ' app/controllers.py\n')
        mox.VerifyAll()
    finally:
        mox.UnsetStubs()
        sys.stdout = sys.__stdout__

def test_locate_non_recursive():
    mox = Mox()

    old_glob = io.glob
    io.glob = mox.CreateMockAnything()

    base_path = '../to/project'
    full_path = '/full/path/to/project'

    class MyFs(io.FileSystem):
        stack = []
        abspath = mox.CreateMockAnything()

    io.glob('%s/*match*.py' % full_path)
    MyFs.abspath(base_path).AndReturn(full_path)
    mox.ReplayAll()
    try:
        MyFs.locate(base_path, '*match*.py', recursive=False)
        mox.VerifyAll()
    finally:
        mox.UnsetStubs()
        io.glob = old_glob

def test_locate_recursive():
    mox = Mox()

    base_path = '../to/project'
    full_path = '/full/path/to/project'

    class MyFs(io.FileSystem):
        stack = []
        abspath = mox.CreateMockAnything()
        walk = mox.CreateMockAnything()

    io.glob('%s/*match*.py' % full_path)
    MyFs.abspath(base_path).AndReturn(full_path)

    walk_list = [
        (None, None, ['file1.py', 'file2.jpg']),
        (None, None, ['path1/file3.png', 'path1/file4.html'])
    ]
    MyFs.walk(full_path).AndReturn(walk_list)

    mox.ReplayAll()
    try:
        MyFs.locate(base_path, '*match*.py', recursive=True)
        mox.VerifyAll()
    finally:
        mox.UnsetStubs()

def test_mkdir_success():
    mox = Mox()

    mox.StubOutWithMock(io, 'os')

    class MyFs(io.FileSystem):
        pass

    io.os.makedirs('/make/all/those/subdirs')

    mox.ReplayAll()
    try:
        MyFs.mkdir('/make/all/those/subdirs')
        mox.VerifyAll()
    finally:
        mox.UnsetStubs()

def test_mkdir_ignore_dirs_already_exists():
    mox = Mox()

    mox.StubOutWithMock(io, 'os')
    mox.StubOutWithMock(io.os, 'path')

    class MyFs(io.FileSystem):
        pass

    oserror = OSError()
    oserror.errno = 17

    io.os.makedirs('/make/all/those/subdirs').AndRaise(oserror)
    io.os.path.isdir('/make/all/those/subdirs').AndReturn(True)

    mox.ReplayAll()
    try:
        MyFs.mkdir('/make/all/those/subdirs')
        mox.VerifyAll()
    finally:
        mox.UnsetStubs()

def test_mkdir_raises_on_oserror_errno_not_17():
    mox = Mox()

    mox.StubOutWithMock(io, 'os')
    mox.StubOutWithMock(io.os, 'path')

    class MyFs(io.FileSystem):
        pass

    oserror = OSError()
    oserror.errno = 0

    io.os.makedirs('/make/all/those/subdirs').AndRaise(oserror)

    mox.ReplayAll()
    try:
        assert_raises(OSError, MyFs.mkdir, '/make/all/those/subdirs')
        mox.VerifyAll()
    finally:
        mox.UnsetStubs()

def tes_mkdir_raises_on_oserror_errno_not_17():
    mox = Mox()

    mox.StubOutWithMock(io, 'os')
    mox.StubOutWithMock(io.os, 'path')

    class MyFs(io.FileSystem):
        pass

    oserror = OSError()
    oserror.errno = 0

    io.os.makedirs('/make/all/those/subdirs').AndRaise(oserror)

    mox.ReplayAll()
    try:
        assert_raises(OSError, MyFs.mkdir, '/make/all/those/subdirs')
        mox.VerifyAll()
    finally:
        mox.UnsetStubs()

def tes_mkdir_raises_when_path_is_not_a_dir():
    mox = Mox()

    mox.StubOutWithMock(io, 'os')
    mox.StubOutWithMock(io.os, 'path')

    class MyFs(io.FileSystem):
        pass

    oserror = OSError()
    oserror.errno = 17

    io.os.makedirs('/make/all/those/subdirs').AndRaise(oserror)
    io.os.isdir('/make/all/those/subdirs').AndReturn(False)
    mox.ReplayAll()
    try:
        assert_raises(OSError, MyFs.mkdir, '/make/all/those/subdirs')
        mox.VerifyAll()
    finally:
        mox.UnsetStubs()




#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from setuptools import setup, find_packages
from sponge import __version__

setup(name='Sponge',
    version=__version__,
    description='A web framework aiming to get things dry, ' \
                'built on top of CherryPy and Genshi',
    author='Gabriel Falcao',
    author_email='gabriel@nacaolivre.org',
    url='http://gnu.gabrielfalcao.com/sponge',
    entry_points={
        'console_scripts': [
            'bob = sponge.bob:run',
        ]
    },
    packages=find_packages(),
    include_package_data = True,
    package_data = {
        'sponge': ['data/project.zip'],
    },
)



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
# Copyright (C) 2009 Bernardo Heynemann <heynemann@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from os.path import abspath, dirname, join, split
from nose.tools import assert_equals
from sponge.core.io import FileSystem

def test_abspath():
    fs = FileSystem()
    p = fs.abspath(".")
    p2 = abspath(".")

    assert p == p2

def test_current_dir_with_join():
    fs = FileSystem()
    got = fs.current_dir("etc")
    expected = join(abspath("."), "etc")

    assert_equals(got, expected)

def test_current_dir_without_join():
    fs = FileSystem()
    got = fs.current_dir()
    expected = abspath(".")

    assert_equals(got, expected)

def test_join():
    fs = FileSystem()
    p = fs.join(fs.abspath("."), "test")
    p2 = join(abspath("."), "test")

    assert p == p2, "Expected:\n%r\nGot:\n%r" % (p2, p)

def test_dirname():
    fs = FileSystem()
    p = fs.dirname(fs.abspath("."))
    p2 = dirname(abspath("."))

    assert p == p2, "Expected:\n%r\nGot:\n%r" % (p2, p)

def test_recursive_locate():
    fs = FileSystem()
    files = fs.locate(path=abspath(join(dirname(__file__), "files_to_locate")), match="*.txt", recursive=True)

    assert files
    assert isinstance(files, list)
    assert len(files) == 2
    assert split(files[0])[-1] == "test.txt"
    assert split(files[1])[-1] == "test2.txt"
    assert split(split(files[1])[0])[-1] == "sub"

def test_non_recursive_locate():
    fs = FileSystem()
    files = fs.locate(path=abspath(join(dirname(__file__), "files_to_locate")), match="*.txt", recursive=False)

    assert files
    assert isinstance(files, list)
    assert len(files) == 1
    assert split(files[0])[-1] == "test.txt"

def test_open_non_abspath():
    fs = FileSystem()
    assert fs.open('tests/functional/data/some.txt', 'r').read() == 'some text here!\n'

def test_open_abspath():
    fs = FileSystem()
    assert fs.open(abspath('./tests/functional/data/some.txt'), 'r').read() == 'some text here!\n'

def test_open_raw_non_abspath():
    fs = FileSystem()
    assert fs.open_raw('tests/functional/data/some.txt', 'r').read() == 'some text here!\n'

def test_open_raw_abspath():
    fs = FileSystem()
    assert fs.open_raw(abspath('./tests/functional/data/some.txt'), 'r').read() == 'some text here!\n'



#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
import os
import cherrypy
from sponge.helpers import image

images = os.path.abspath(os.path.join(os.path.dirname(__file__), 'data'))

def test_jpeg():
    cherrypy.config['image.dir'] = images
    got = image.jpeg('2823371.jpg')
    assert isinstance(got, basestring), 'Expected a string, got %r' % got




# -*- coding: utf-8 -*-
import sys
from StringIO import StringIO
from mox import Mox
from nose.tools import assert_equals
from utils import assert_raises
from sponge import core
from sponge.core import SpongeConfig, ConfigValidator

config_dict = {
    'run-as': 'wsgi',
    'host': '0.0.0.0',
    'port': 80,
    'autoreload': False,
    'application': {
        'path': 'path/to/project',
        'template-dir': '/path/to/project/templates',
        'image-dir': '/path/to/project/images',
        'static': {
            '/media': 'my/media'
        },
        'classes': {
            'SomeController': '/'
        }
    },
    'extra': {
        'something': '/bla/foo/bar'
    }
}

def test_takes_dict_on_creation():
    cf = ConfigValidator({})
    assert_raises(TypeError, SpongeConfig, None, cf,
                  exc_pattern=r'SpongeConfig parameter 1 ' \
                  'must be a dict, got None.')
    assert_raises(TypeError, SpongeConfig, 5, cf,
                  exc_pattern=r'SpongeConfig parameter 1 ' \
                  'must be a dict, got 5.')

def test_can_set_setting():
    d = {}
    cf = ConfigValidator({})
    sp = SpongeConfig(d, cf)
    sp.set_setting('my-setting', 'my-value')
    assert_equals(d['my-setting'], 'my-value')

def test_setup_all_takes_string_path():
    d = {}
    cf = ConfigValidator({})
    sp = SpongeConfig(d, cf)
    assert_raises(TypeError, sp.setup_all, tuple(),
                  exc_pattern=r'SpongeConfig.setup_all takes a ' \
                  'string, got ().')
    assert_raises(TypeError, sp.setup_all, None,
                  exc_pattern=r'SpongeConfig.setup_all takes a ' \
                  'string, got None.')
    assert_raises(TypeError, sp.setup_all, 10,
                  exc_pattern=r'SpongeConfig.setup_all takes a ' \
                  'string, got 10.')

def test_setup_all_path_must_be_absolute():
    d = {}
    cf = ConfigValidator({})
    sp = SpongeConfig(d, cf)
    assert_raises(TypeError, sp.setup_all, 'relative/path/',
                  exc_pattern=r'SpongeConfig.setup_all takes a ' \
                  'absolute path, got relative/path/.')

def test_can_setup_all_without_routes_attr():
    mox = Mox()
    d = {}
    class_loader = core.ClassLoader
    cherrypy = core.cherrypy
    core.ClassLoader = mox.CreateMockAnything()
    core.cherrypy = mox.CreateMockAnything()
    core.cherrypy.tree = mox.CreateMockAnything()

    cloader_mock = mox.CreateMockAnything()
    core.ClassLoader('/absolute/path/path/to/project').AndReturn(cloader_mock)

    class_mock = lambda: 'should_be_some_controller_instance'
    class_mock.__name__ = 'class_mock_lambda'

    cloader_mock.load('SomeController').AndReturn(class_mock)

    core.cherrypy.tree.mount(root='should_be_some_controller_instance',
                             script_name='/', config={
                                 '/media': {
                                     'tools.staticdir.dir': '/absolute/path/my/media',
                                     'tools.staticdir.on': True
                                 }
                             })

    cf = core.ConfigValidator(config_dict)
    sp = core.SpongeConfig(d, cf)
    sp.set_setting = mox.CreateMockAnything()

    sp.set_setting('server.socket_port', 80)
    sp.set_setting('server.socket_host', '0.0.0.0')
    sp.set_setting('tools.sessions.on', True)
    sp.set_setting('tools.sessions.timeout', 60)
    sp.set_setting('tools.encode.on', True)
    sp.set_setting('tools.encode.encoding', 'utf-8')
    sp.set_setting('tools.trailing_slash.on', True)
    sp.set_setting('sponge', config_dict)
    sp.set_setting('sponge.root', '/absolute/path')
    sp.set_setting('sponge.extra', config_dict['extra'])
    sp.set_setting('template.dir', '/path/to/project/templates')
    sp.set_setting('image.dir', '/path/to/project/images')
    mox.ReplayAll()
    sys.stderr = StringIO()
    try:
        sp.setup_all('/absolute/path/')
        assert_equals(sys.stderr.getvalue(),
                      '\nWARNING: The class %s has no routes\n' % class_mock.__name__)
        mox.VerifyAll()
    finally:
        core.ClassLoader = class_loader
        core.cherrypy = cherrypy
        sys.stderr = sys.__stderr__

def test_can_setup_all_without_routes_dict():
    mox = Mox()
    d = {}
    class_loader = core.ClassLoader
    cherrypy = core.cherrypy
    core.ClassLoader = mox.CreateMockAnything()
    core.cherrypy = mox.CreateMockAnything()
    core.cherrypy.tree = mox.CreateMockAnything()

    cloader_mock = mox.CreateMockAnything()
    core.ClassLoader('/absolute/path/path/to/project').AndReturn(cloader_mock)

    class_mock = mox.CreateMockAnything()
    class_mock.__routes__ = 'blabla'

    cloader_mock.load('SomeController').AndReturn(class_mock)
    class_mock().AndReturn('should_be_some_controller_instance')

    core.cherrypy.tree.mount(root='should_be_some_controller_instance',
                             script_name='/', config={
                                 '/media': {
                                     'tools.staticdir.dir': '/absolute/path/my/media',
                                     'tools.staticdir.on': True
                                 }
                             })

    cf = core.ConfigValidator(config_dict)
    sp = core.SpongeConfig(d, cf)
    sp.set_setting = mox.CreateMockAnything()

    sp.set_setting('server.socket_port', 80)
    sp.set_setting('server.socket_host', '0.0.0.0')
    sp.set_setting('tools.sessions.on', True)
    sp.set_setting('tools.sessions.timeout', 60)
    sp.set_setting('tools.encode.on', True)
    sp.set_setting('tools.encode.encoding', 'utf-8')
    sp.set_setting('tools.trailing_slash.on', True)
    sp.set_setting('sponge', config_dict)
    sp.set_setting('sponge.root', '/absolute/path')
    sp.set_setting('sponge.extra', config_dict['extra'])
    sp.set_setting('template.dir', '/path/to/project/templates')
    sp.set_setting('image.dir', '/path/to/project/images')

    mox.ReplayAll()
    sys.stderr = StringIO()
    try:
        sp.setup_all('/absolute/path/')
        assert_equals(sys.stderr.getvalue(), '')
        mox.VerifyAll()
    finally:
        core.ClassLoader = class_loader
        core.cherrypy = cherrypy
        sys.stderr = sys.__stderr__

def test_setup_all_fails_on_import():
    mox = Mox()
    d = {}
    class_loader = core.ClassLoader
    cherrypy = core.cherrypy
    core.ClassLoader = mox.CreateMockAnything()
    core.cherrypy = mox.CreateMockAnything()
    core.cherrypy.tree = mox.CreateMockAnything()

    cloader_mock = mox.CreateMockAnything()
    core.ClassLoader('/absolute/path/path/to/project').AndReturn(cloader_mock)

    class_mock = mox.CreateMockAnything()
    cloader_mock.load('SomeController').AndRaise(Exception('foo error'))

    cf = core.ConfigValidator(config_dict)
    sp = core.SpongeConfig(d, cf)
    mox.ReplayAll()
    try:
        sys.stderr = StringIO()
        assert_raises(SystemExit, sp.setup_all, '/absolute/path/')
        got = sys.stderr.getvalue()
        sys.stderr = sys.__stderr__

        format_args = 'SomeController', \
                      '/absolute/path/path/to/project', \
                      'foo error'
        assert_equals(got, '\nSponge could not find the class %s ' \
                     'at %s, verify if your settings.yml ' \
                     'is configured as well\n%s\n' % format_args)
        mox.VerifyAll()
    finally:
        core.ClassLoader = class_loader
        core.cherrypy = cherrypy

def test_can_setup_all_with_routes():
    mox = Mox()
    d = {}
    class_loader = core.ClassLoader
    cherrypy = core.cherrypy
    core.ClassLoader = mox.CreateMockAnything()
    core.cherrypy = mox.CreateMockAnything()
    core.cherrypy.tree = mox.CreateMockAnything()
    core.cherrypy.dispatch = mox.CreateMockAnything()

    cloader_mock = mox.CreateMockAnything()
    core.ClassLoader('/absolute/path/path/to/project').AndReturn(cloader_mock)

    class_mock = mox.CreateMockAnything()
    class_mock.__name__ = "MyPhotoController"
    class_mock.__routes__ = [
        ('show_photos', {
            'route': '/photos',
            'method': 'list_photos',
        }),
        (None, {
            'route': '/photo/:id/edit',
            'method': 'edit',
        })
    ]
    controller_mock = mox.CreateMockAnything()
    cloader_mock.load('SomeController').AndReturn(class_mock)
    class_mock().AndReturn(controller_mock)
    class_mock().AndReturn(controller_mock)

    routes_mock = mox.CreateMockAnything()
    core.cherrypy.dispatch.RoutesDispatcher().AndReturn(routes_mock)

    routes_mock.connect(name='show_photos',
                        controller=controller_mock,
                        route='/photos',
                        action='list_photos')
    routes_mock.connect(name='MyPhotoController.edit',
                        controller=controller_mock,
                        route='/photo/:id/edit',
                        action='edit')

    core.cherrypy.tree.mount(root=None, config={
        '/': {
            'request.dispatch': routes_mock
        },
        '/media': {
            'tools.staticdir.dir': '/absolute/path/my/media',
            'tools.staticdir.on': True
        }
    })

    cf = core.ConfigValidator(config_dict)
    sp = core.SpongeConfig(d, cf)
    mox.ReplayAll()
    try:
        sp.setup_all('/absolute/path/')
        mox.VerifyAll()
    finally:
        core.ClassLoader = class_loader
        core.cherrypy = cherrypy

def test_boot():
    mox = Mox()
    d = {}
    class_loader = core.ClassLoader
    cherrypy = core.cherrypy
    core.ClassLoader = mox.CreateMockAnything()
    core.cherrypy = mox.CreateMockAnything()
    core.cherrypy.tree = mox.CreateMockAnything()


    module_mock0 = mox.CreateMockAnything()
    module_mock0.my_function()
    cloader_mock0 = mox.CreateMockAnything()
    cloader_mock0.get_module().AndReturn(module_mock0)

    core.ClassLoader('/path/to/boot/file.py').AndReturn(cloader_mock0)


    cloader_mock = mox.CreateMockAnything()
    core.ClassLoader('/absolute/path/path/to/project').AndReturn(cloader_mock)

    class_mock = mox.CreateMockAnything()
    class_mock.__routes__ = 'blabla'

    cloader_mock.load('SomeController').AndReturn(class_mock)
    class_mock().AndReturn('should_be_some_controller_instance')

    core.cherrypy.tree.mount(root='should_be_some_controller_instance',
                             script_name='/', config={
                                 '/media': {
                                     'tools.staticdir.dir': '/absolute/path/my/media',
                                     'tools.staticdir.on': True
                                 }
                             })

    my_config = config_dict.copy()
    my_config['application']['boot'] = {
        'path': '/path/to/boot/file.py',
        'callable': 'my_function'
    }
    cf = core.ConfigValidator(my_config)

    sp = core.SpongeConfig(d, cf)
    sp.set_setting = mox.CreateMockAnything()

    sp.set_setting('server.socket_port', 80)
    sp.set_setting('server.socket_host', '0.0.0.0')
    sp.set_setting('tools.sessions.on', True)
    sp.set_setting('tools.sessions.timeout', 60)
    sp.set_setting('tools.encode.on', True)
    sp.set_setting('tools.encode.encoding', 'utf-8')
    sp.set_setting('tools.trailing_slash.on', True)
    sp.set_setting('sponge', config_dict)
    sp.set_setting('sponge.root', '/absolute/path')
    sp.set_setting('sponge.extra', config_dict['extra'])
    sp.set_setting('template.dir', '/path/to/project/templates')
    sp.set_setting('image.dir', '/path/to/project/images')


    mox.ReplayAll()
    sys.stderr = StringIO()
    try:
        sp.setup_all('/absolute/path/')
        assert_equals(sys.stderr.getvalue(), '')
        mox.VerifyAll()
    finally:
        core.ClassLoader = class_loader
        core.cherrypy = cherrypy
        sys.stderr = sys.__stderr__




#!/usr/bin/env python
# -*- coding: utf-8; -*-
#
# Copyright (C) 2009 Gabriel Falcão <gabriel@nacaolivre.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from mox import Mox
from nose.tools import assert_equals
from utils import assert_raises

from sponge.core import io

def test_class_loader_takes_a_string_path_raises_with_number():
    assert_raises(TypeError, io.ClassLoader, 5,
                  exc_pattern=r'ClassLoader takes a string ' \
                  'as path parameter, got 5.')

def test_class_loader_takes_a_string_path_raises_with_none():
    assert_raises(TypeError, io.ClassLoader, None,
                  exc_pattern=r'ClassLoader takes a string ' \
                  'as path parameter, got None.')

def test_class_loader_loads_from_file():
    mox = Mox()

    mox.StubOutWithMock(io, 'os')
    mox.StubOutWithMock(io.sys, 'path')
    io.os.path = mox.CreateMockAnything()
    io.__import__ = mox.CreateMockAnything()

    class_dir = '/full/path/to/module/or'
    class_file = 'file.py'
    class_path = '%s/%s' % (class_dir, class_file)

    io.os.path.isdir(class_path).AndReturn(False)

    io.os.path.split(class_path).AndReturn((class_dir, class_file))
    io.os.path.splitext(class_file).AndReturn(('file', '.py'))

    io.sys.path.append(class_dir)
    io.sys.path.pop()
    module_mock = mox.CreateMockAnything()
    module_mock.ClassIWantToLoad = 'should_be_expected_class'
    io.__import__('file').AndReturn(module_mock)

    mox.ReplayAll()

    try:
        cl = io.ClassLoader(class_path)
        assert_equals(cl.load('ClassIWantToLoad'), 'should_be_expected_class')
        mox.VerifyAll()
    finally:
        io.__import__ = __import__
        mox.UnsetStubs()

def test_class_loader_loads_from_module():
    mox = Mox()

    mox.StubOutWithMock(io, 'os')
    mox.StubOutWithMock(io.sys, 'path')

    io.os.path = mox.CreateMockAnything()
    io.__import__ = mox.CreateMockAnything()

    class_path = '/full/path/to/module/'
    module_name = 'module'
    module_dir = '/full/path/to/'

    io.os.path = mox.CreateMockAnything()

    io.os.path.isdir(class_path).AndReturn(True)

    io.os.path.split(class_path.rstrip('/')).AndReturn((module_dir, module_name))

    io.sys.path.append(module_dir)
    io.sys.path.pop()

    module_mock = mox.CreateMockAnything()
    module_mock.ClassIWantToLoad = 'should_be_expected_class'
    io.__import__('module').AndReturn(module_mock)

    mox.ReplayAll()

    try:
        cl = io.ClassLoader(class_path)
        assert_equals(cl.load('ClassIWantToLoad'), 'should_be_expected_class')
        mox.VerifyAll()
    finally:
        io.__import__ = __import__
        mox.UnsetStubs()

def test_get_module():
    mox = Mox()

    mox.StubOutWithMock(io, 'os')
    mox.StubOutWithMock(io.sys, 'path')

    io.os.path = mox.CreateMockAnything()
    io.__import__ = mox.CreateMockAnything()

    class_path = '/full/path/to/module/'
    module_name = 'module'
    module_dir = '/full/path/to/'

    io.os.path = mox.CreateMockAnything()

    io.os.path.isdir(class_path).AndReturn(True)

    io.os.path.split(class_path.rstrip('/')).AndReturn((module_dir, module_name))

    io.sys.path.append(module_dir)
    io.sys.path.pop()

    module_mock = mox.CreateMockAnything()
    module_mock.ClassIWantToLoad = 'should_be_expected_class'
    io.__import__('module').AndReturn(module_mock)

    mox.ReplayAll()

    try:
        cl = io.ClassLoader(class_path)
        assert_equals(cl.get_module(), module_mock)
        mox.VerifyAll()
    finally:
        io.__import__ = __import__
        mox.UnsetStubs()

